	PROGRAM VERIFY_Q
	IMPLICIT NONE
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&VERIFY_Q\&
C .nf
C .x VERIFY_Q>Defined
C Source:VERIFY_Q.MOD
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 22-JUN-1988
C Revision level     :1.4
C  
C .C ;Formal Parameter List
C Receives:
C  
C Returns:
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C    Q_SECTION.QUE                                                                
C                               READ                                              
C .X Q_SECTION.QUE>READ
C  
C    QUEUE_DUMP.DAT                                                           
C                               CREATE
C .X QUEUE_DUMP.DAT>CRATE
C  
C Other modules referenced:
C  
C .X DUMPIT>Referenced
C    DUMPIT                     CONTAINED IN THIS PROGRAM AND IS RESPONSIBLE      
C                               FOR ACTUALLY EXTRACTING THE MESSAGES FROM THE     
C                               FILE.                                             
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This program is run to verify the integrity of a QUEUE file after
c a system crash or other restart.  This program is run prior to the 
c installation of the queue file as a known global section by the
c QUEUE_CHKR process.  The algothrim that is used is as follows:
c .list
c .le;Verify that the queue header is correct.
c .le;Allocate a bit map for the number of message blocks
c that are in the queue.
c .le;Traverse the free list chain and mark all bits
c in the bit map that are in the free list.
c .le;For each message id, scan the RNA and the forward
c link and verify that the links are valid by the following
c test:
c .list
c .le;The forward/backward threads point to valid blocks
c within the queue.
c .le;The forward/backward threads point to blocks that are
c not on the free list or some other message id list 
c (ie: the bit is not set in our bit map).
c .le;Mark the block as used by setting the bit in the bit map
c for this block.
c .els
c .els
c 	In the event that a message block points to an invalid
c block, or a block that has already been used, then
c terminate the list (if this is a RNA message, simply remove
c the RNA message).  Modify the message counter to reflect the
c true number of messages that are present for this message id.
c 	For each message that is scanned, ensure that there are sufficient
c blocks for the number bytes specified in the header for the message.
c 	After all messages have been scanned, update the used blocks
c in the header and scan the bit map and restore any unthreaded blocks
c back onto the free list.
C 	This program is run with the DCL command: "RUN VERIFY_Q".
C The only input to the program is the QUEUE file name, which
C if not specified is: "Q_SECTION.QUE".
C 
C end.doc ****************************** end.doc  
C
C REVISION HISTORY
c
c 15-JUL-1987  E. Lakia
c	Modified to allow the entry of the queue name so that this
c program can be used on a system with more than a single queue.
C
C 19-MAY-1987 E. LAKIA
C	Modified to make sure that the queue section is not installed
c when the verification is done.
c 
C 18-MAY-1987  1.1 E. Lakia
c	Modified free_list to check for a zero free list pointer.
c  If true, then reset the free list pointer.
c

C
C THE STRUCTURE OF THE QUEUE WITHIN THE DISK FILE IS:
C
C
C HEDBLKS=QHD$K_SIZ/512 blocks for the queue header,
c			(note: currently TWO allocated).
c MIDBLKS=(QHD$L_AID*MID$K_LEN)/511 rounded upward to a whole block
c		are the message id listhead blocks
C MSGBLKS= QHD$L_SIZ blocks allocated for the actual messages.
c 
C
C
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(MIDDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QHDDEF)/LIST'
	INCLUDE '($SSDEF)'	! SYSTEM SERVICES DEFINITIONS
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	RECORD/QHDDEF/QHD	! QUEUE HEADER
	INTEGER*4 QSZ		! QUEUE FILE NAME LENGTH
	CHARACTER*80 QUENAME	! NAME OF THE QUEUE FILE TO UNLOAD
	INTEGER*4 IOSTAT	! RMS OPEN STATUS
	INTEGER*4 QUELUN	! QUEUE FILE LUN
	PARAMETER(QUELUN=1)
	INTEGER*4 IDBLKADDR(2)	! RETURN ADDRESS WHERE LISTHEAD STORAGE ALLOCATED
	INTEGER*4 VERIFY_IT	! SUBROUTINE TO VERIFY QUEUE (TO FAKE FORTRAN)
	INTEGER*4 I		! LOOP VARIABLE
	INTEGER*4 IDBLOCKS	! NUMBER OF BLOCKS IN MESSAGE IDS
	INTEGER*4 SYS$EXPREG	! EXPAND REGION
	INTEGER*4 BLOCK(128)	! SINGLE DISK BLOCK
	INTEGER*4 LASTREC	! LAST RECORD NUMBER
	INTEGER*4 BITMAPSIZE	! SIZE OF BITMAP
	INTEGER*4 BITMAPBLKS	! NUMBER OF BLOCKS IN BITMAP
	INTEGER*4 BITMAPADDR(2) ! WHERE BIT MAP IS MAPPED IN OUR PROCESS
	INTEGER*4 IMAX		! MAXIMUM OFFSET FOR VALID MESSAGE IDS LHD'S
	INTEGER*4 Q_BLOCK(2)	! Q BLOCK ADDRESS	
	INTEGER*4 ATTACH_Q	! ATTACH TO Q (TO SEE IF PRESENT IN SYSTEM)
	CHARACTER*16 SECTION_NAME  ! Current section name for queue
	INTEGER*2 SNLTH		! Length of queue section name

C
C Get user options, page file name and the queue section name
c
	WRITE(*,100)
100	FORMAT('$Enter queue page file filename ',/,
	1	'   [Default: MA_Q$PROD:Q_SECTION.QUE; ]: ')
	READ(*,200,END=210)QSZ,QUENAME(1:QSZ)
200	FORMAT(Q,A<QSZ>)
C
210	IF(QSZ .EQ. 0)THEN
	  QUENAME='MA_Q$PROD:Q_SECTION.QUE;'
	  QSZ=24
	ENDIF
c
	WRITE(*,250)
250	FORMAT('$Enter the QUEUE name ',
	1	'[Default is logical- GBL$MA_Q_NAME]: ')
	READ(*,260,END=270)SNLTH,SECTION_NAME(1:SNLTH)
260	FORMAT(Q,A<SNLTH>)
C
C
C FIRST OPEN THE QUEUE FILE
C
270	CONTINUE
	OPEN(UNIT=QUELUN,NAME=QUENAME(1:QSZ),
	1	TYPE='OLD',
	1	RECORDSIZE=128,
	1	ACCESS='DIRECT',
	1	IOSTAT=IOSTAT,
	1	SHARED,
	1	ERR=92000)
C
C MAKE SURE THAT THE QUEUE IS NOT CURRENTLY INSTALLED
C
	IF(SNLTH .EQ. 0)THEN		! Default queue?
		SYS_STATUS=ATTACH_Q(Q_BLOCK)	! Try to attach to verify
	ELSE				! queue name specified, use it
		SYS_STATUS=ATTACH_Q(Q_BLOCK,SECTION_NAME(1:SNLTH))
	ENDIF
C
	IF(SYS_STATUS)THEN		! If attach good, then q is installed
		TYPE *,' ATTACH_Q STATUS IS GOOD, IE: ',SYS_STATUS
		TYPE *,' THE QUEUE IS STILL INSTALLED IN THE SYSTEM'
		TYPE *,' REMOVE THE QUEUE FIRST'
		CALL SYS$EXIT(%VAL(0))
	ENDIF
C
C READ THE QUEUE HEADER (NOTE: we assume queue header is only two blocks)
C
800	CONTINUE
	READ(QUELUN'1)(QHD.QHDDEF_ALL(I),I=1,512)
	READ(QUELUN'2)(QHD.QHDDEF_ALL(I),I=513,QHD$K_SIZ)
	TYPE *,' Queue size: ',QHD.QHD$L_SIZ
	TYPE *,' Number of allocated message ids: ',QHD.QHD$L_AID
	TYPE *,' Number of used message ids: ',QHD.QHD$L_IDC
	TYPE *,' Number of free blocks: ',QHD.QHD$L_FREE
	TYPE *,' Number of used blocks: ',QHD.QHD$L_SIZ-QHD.QHD$L_FREE
C
	WRITE(*,300)QHD.QHD$Q_FLS
300	FORMAT(' FREE LIST HEAD: ',2(1X,Z8.8))
	IDBLOCKS= ((QHD.QHD$L_AID*MID$K_LEN)+511)/512
	TYPE *,' Number of ID blocks: ',IDBLOCKS
	LASTREC=QHD.QHD$L_SIZ ! Largest valid record
	TYPE *,' Reading last record in queue: ',LASTREC

C
C verify that the queue header is in tact
c
	READ(QUELUN'LASTREC,ERR=93000)BLOCK  ! Make sure can read last block
	IF(QHD.QHD$L_IDC .GT. QHD.QHD$L_AID)THEN ! number used > allocated?
		TYPE *,' Number message ids used greater than allocated'
		TYPE *,' QUEUE HEADER is corrupt'
		CALL SYS$EXIT(%VAL(0))
	ENDIF
C
	IF(QHD.QHD$L_FREE .GT. QHD.QHD$L_SIZ)THEN ! More free than allocated?
		TYPE *,' Number of free block greater than allocated'
		TYPE *,' QUEUE HEADER is corrupt'
		CALL SYS$EXIT(%VAL(0))
	ENDIF
C
	IF(QHD.CLOCK_CNT .GT. 19 .OR.
	1	QHD.CLOCK_CNT .LT. 0)THEN ! Lock id char. count too large?
		TYPE *,' Lock name for queue is corrupt'
		TYPE *,' QUEUE HEADER is corrupt'
		CALL SYS$EXIT(%VAL(0))	! BAD STATUS
	ENDIF
C
	IMAX=(QHD.QHD$L_AID *MID$K_LEN)+ QHD$K_SIZ ! MAXIMUM LHD OFFSET
	DO I=1,128	! 128 HASH ENTRIES
		IF(QHD.QHD$C_HASH(I) .GT. IMAX .OR.
	1		QHD.QHD$C_HASH(I) .LT. 0)THEN
			TYPE *,' HASH TABLE entry: ',I,' is corrupt'
			TYPE *,' QUEUE HEADER is corrupt'
			CALL SYS$EXIT(%VAL(0))
		ENDIF
	ENDDO
c
	TYPE *,' QUEUE HEADER appears intact, continueing'

c
c CREATE A BIT MAP FOR THE SIZE OF THE QUEUE
C
	BITMAPSIZE=(QHD$K_SIZ+31)/32 	! NUMBER OF LONG WORDS
	BITMAPBLKS=(BITMAPSIZE+511)/512	! NUMBER OF BLOCKS FOR BIT MAP
	SYS_STATUS=SYS$EXPREG(%VAL(BITMAPBLKS),  ! Number of blocks
	1	BITMAPADDR,			! return address
	1	,,)				! put in P0, user access mode
	IF(.NOT. SYS_STATUS)THEN	! What, unable to expand region?
		TYPE *,' Unable to expand region for bit map'
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	ENDIF
c
c call to have the free list checked
c
	CALL FREE_LIST(%VAL(BITMAPADDR(1)),	! BIT MAP
	1	BITMAPSIZE,			! number of words in bitmap
	1	LASTREC,			! LAST VALID BLOCK IN QUEUE
	1	QHD)				! QUEUE HEADER

C
C CREATE A PLACE FOR THE MESSAGE ID LISTHEADS
C
	TYPE *,' Number of message id listhead blocks: ',IDBLOCKS
	SYS_STATUS=SYS$EXPREG(%VAL(IDBLOCKS),  ! Number of blocks
	1	IDBLKADDR,			! return address
	1	,,)				! put in P0, user access mode
	IF(.NOT. SYS_STATUS)THEN	! What, unable to expand region?
		TYPE *,' Unable to expand region for message id list'
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	ENDIF	
c
	WRITE(*,400)IDBLKADDR
400	FORMAT(' List heads mapped at: ',2(1X,Z8.8))
C
C SINCE WE NOW HAVE A UNSUPPORTED FORTRAN AREA TO READ THE 
C LISTHEAD INTO, WE WILL CALL A SUBROUTINE TO FAKE FORTRAN OUT.
C
	SYS_STATUS=VERIFY_IT(QUELUN,
	1	QHD,
	1	%VAL(IDBLKADDR(1)),	! fake way for easy block reading
	1	%VAL(IDBLKADDR(1)),	!   "   "   "  easy record using
	1	QHD.QHD$L_AID,		! number of message ids
	1	LASTREC,		 ! largest valid record 
	1	%VAL(BITMAPADDR(1)))	! BIT MAP
c
C Now scan the bit map and relink any blocks that have
c been left around because of the early termination of message
c id chains or the free list chains.
C 
	CALL RECLAIM(QHD,		! Queue header
	1	%VAL(BITMAPADDR(1)),	! bit map
	1	QHD$K_SIZ/512+IDBLOCKS+1, ! first message block
	1	LASTREC)		! last message block
C
C Write back the queue header in case the listhead 
c has changed
c
	IF(QHD.QHD$W_VERSION .NE. QHD$$VERSION)THEN
		TYPE *,'Queue version numbers are not compatible'
		TYPE *,' Current queue version: ',QHD.QHD$W_VERSION
		TYPE *,' Current release version: ',QHD$$VERSION
		CALL SYS$EXIT(%VAL(0))		! oops
	ENDIF
c
	QHD.QHD$L_STAT=QHD$M_SHUTDOWN	! mark as shutdown
	WRITE(QUELUN'1)(QHD.QHDDEF_ALL(I),I=1,512)
	WRITE(QUELUN'2)(QHD.QHDDEF_ALL(I),I=513,QHD$K_SIZ)
C
	CLOSE(UNIT=QUELUN)
	CALL SYS$EXIT(%VAL(SS$_NORMAL))

C
C EXIT/ERRORS
C
90000	CALL SYS$EXIT(%VAL(SS$_NORMAL))
92000	TYPE *,' OPEN ERROR '
	CALL FTN_IOSTAT
	CALL SYS$EXIT(%VAL(IOSTAT))
c
c Can't read last record of queue
c
93000	TYPE *,' Unable to read last block of queue'
	TYPE *,' QUEUE header is corrupt'
	CALL FTN_IOSTAT
	CALL SYS$EXIT(%VAL(0))
	END
	INTEGER*4 FUNCTION VERIFY_IT(QUELUN,QHD,MID,BLOCKS,
	1	MIDCNT,MAXREC,BITMAP)
	IMPLICIT NONE
C
c This subroutine is responsible for the scanning of each of the 
c message id threads and ensuring that they are in tact.  If 
c any thread is invalid, the thread is terminated at the last
c valid message and the number of messages for that list
c head is modified to reflect the shortened thread.  The
c number of non volatile messages are also kept track of
c for update into the message id listhead.  The
c list heads are validated first before the scan is done and
c if any of the list heads are not valid, the message id
c list head is reset.
c
	INCLUDE '($SSDEF)'	! SYSTEM SERVICES DEFINITIONS
C
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(HDRDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(MIDDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QHDDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QSTDEF)/LIST'
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	INTEGER*4 MIDCNT	! number of message ids
	RECORD/QHDDEF/QHD	! QUEUE HEADER
	RECORD/MIDDEF/MID(MIDCNT) ! Message ids
c
	INTEGER*4 BLOCKS(128,*)  ! easy way to read the blocks
	INTEGER*4 QUELUN	! queue section file lun
	INTEGER*4 I,J,K		! index variables
	RECORD/QSTDEF/BLOCK	! Message buffer in queue
	INTEGER*4 MAXREC	! largest valid record number
	INTEGER*4 BLOCK_COUNT	! number of message id blocks
	INTEGER*4 OFF_FRST_MSG	! offset from beginning of queue for
				! first message block
	INTEGER*4 FLINK		! Forward link pointer
	INTEGER*4 MFLINK	! Message blocks forward link pointer
	INTEGER*4 NEXT_DATA_OFF ! relative offset to next data block of mesg.
	INTEGER*4 NEXT_MSG_LINK ! relative offset to next mesg for the id
	INTEGER*4 FLINK_OFF	! forward offset to next mesg
	INTEGER*4 NEXT_BLOCK	! next block of a message (blocks 3-n)
	INTEGER*4 MID_LHD	! address of current message id listhead
	INTEGER*4 MIDOFF	! offset of current message id listhead
	INTEGER*4 LAST_DATA_LINK ! list head of the data blocks threaded
				! to the first block of a message.
	INTEGER*4 BITMAP(*)	! bit map of used blocks
	LOGICAL*4 IFSET		! Returns true/false if bit set/clear
	INTEGER*4 FNDCNT	! Number of message id list heads found
	LOGICAL LHDOK		! logical denoting list head ok
	INTEGER*4 MSGCNT	! number of messages found for a message lhd
	INTEGER*4 SAVLNK	! save previous message link

C
C read the message ids into our records (BLOCKS and MID
C are equivalenced due to the way the calling stack was set up).
C
D	TYPE *,' Number of message ids allocated: ',MIDCNT
D	TYPE *,' Largest valid record number: ',MAXREC
	K=0
	BLOCK_COUNT=((MIDCNT*MID$K_LEN)+511)/512 ! # message id blocks
	DO I=(QHD$K_SIZ/512)+1,BLOCK_COUNT+(QHD$K_SIZ/512)
d		TYPE *,' Reading record: ',I
		K=K+1
d		WRITE(*,50)%LOC(BLOCKS(1,K))
d50		FORMAT(' Buffer: ',Z8.8)
		READ(QUELUN'I)(BLOCKS(J,K),J=1,128)
	ENDDO
c

C
C Scan the message id list heads and verify that the list head is valid.
c If it is, then traverse the messge id threads.
c
	OFF_FRST_MSG= ((BLOCK_COUNT-1)*512)+QHD$K_SIZ  ! offset to start 
C							of message blocks
	WRITE(*,80)OFF_FRST_MSG
80	FORMAT(' Message blocks start at offset: ',Z8.8)
	OFF_FRST_MSG= (OFF_FRST_MSG/512)+1 ! Minimum block number
	TYPE *,' First message blocks start at block: ',OFF_FRST_MSG
C
	DO 10000 I=1,MIDCNT
	  MIDOFF=QHD$K_SIZ+((I-1)*MID$K_LEN) ! OFFSET TO MESSAGE ID
D	  TYPE *,' ID SLOT: ',I
D	  WRITE(*,90)MID(I).MID$L_STAT
D90	  FORMAT(' MID$L_STAT: ',Z8.8)
c
c 	  check the message id list head
c
	  LHDOK=.TRUE.	! Assume list head ok
	  IF((MID(I).MID$L_STAT .AND. MID$M_DEL) .NE. 0)THEN ! ID IN USE?
				 ! NO, does anyone's
c				  hash chain point to deleted message id
		DO K=1,128	! 128 hash keys
		  IF(QHD.QHD$C_HASH(K) .EQ. MIDOFF)THEN
			TYPE *,' Hash key list threads a deleted message id'
			TYPE *,' Some message ids maybe unable to be found'
			QHD.QHD$C_HASH(K)=0  ! Zero hash chain
		  ENDIF
		ENDDO
C
		DO K=1,MIDCNT
		  IF(MID(K).MID$L_HASH .EQ. MIDOFF)THEN
			TYPE *,' Hash chain threads a deleted message id'
			TYPE *,' Some message ids maybe unable to be found'
			MID(K).MID$L_HASH=0 ! ZERO HASH CHAIN
		  ENDIF
		ENDDO
		GO TO 10000
	  ENDIF

C
C	  IN USE MESSAGE ID
C
	  FNDCNT=FNDCNT+1
	  IF(MID(I).MID$W_CNT .GT. MID(I).MID$W_MAX .OR.
	1	MID(I).MID$W_CNT .LT. 0)THEN
		LHDOK=.FALSE.
		TYPE *,' Message id count corrupt'
	        MID(I).MID$W_CNT=25	! Set to a default value
	  endif
c
	  IF(MID(I).MID$W_NVCNT .GT. MID(I).MID$W_CNT .OR.
	1	MID(I).MID$W_NVCNT .GT. MID(I).MID$W_MAX .OR.
	1	MID(I).MID$W_NVCNT .LT. 0)THEN
		TYPE *,' Number of non-volatile messages corrupt'
		LHDOK=.FALSE.
	  ENDIF
C
	  IF(MID(I).MID$L_HASH .GT. 	! Valid thread to next mid lhd?
	1		((BLOCK_COUNT-1)*512)+QHD$K_SIZ)THEN
		TYPE *,' Hash chain invalid '
		TYPE *,' This may eliminate ability to find all ids'
		LHDOK=.FALSE.
		MID(I).MID$L_HASH=0  ! Terminate hash chain
	  ENDIF
C
	  IF(.NOT. LHDOK)THEN
		TYPE *,' Resetting message id list head',MID(I).MID$C_MID
		MID(I).MID$L_FLK=0  ! empty list, point to self
		MID(I).MID$L_BLK=-4	! point to forward list
		MID(I).MID$L_RNA=0	! zero most everything else
		MID(I).MID$L_BRNA=0
		MID(I).MID$W_QSEQ=0
		MID(I).MID$L_STAT=0
		MID(I).MID$L_CPID=0
		MID(I).MID$W_NVCNT=0
		MID(I).MID$W_EFN=0
		MID(I).MID$L_PCB=0
		MID(I).MID$L_IPID=0
		MID(I).MID$W_FILL=0
		GO TO 9000		! Go look at rest of id's
	  ENDIF

c
c valid message id list head, see if any messages
c
	  MSGCNT=0	! init number found
	  IF(MID(I).MID$W_CNT .EQ. 0)GO TO 9000	! ID have any messages?
	  SAVLNK=0	! init saved previous to no previous
	  TYPE *,' Found messages for: ',MID(I).MID$C_MID
	  TYPE *,' Number of messages: ',MID(I).MID$W_CNT
d	  TYPE *,' Number of nonvoltile: ',MID(I).MID$W_NVCNT
c
c	  First see if there is a RNA message
c
	  IF(MID(I).MID$L_RNA .NE. 0)THEN	! must dump RNA first
	    TYPE *,' Message id has an oustanding RNA message'
	    WRITE(*,85)MID(I).MID$L_RNA
85	    FORMAT(' RNA POINTER: ',Z8.8)
	    MSGCNT=MSGCNT+1 ! one more found
	    FLINK_OFF=MID(I).MID$L_RNA+ ! Offset to RNA message
	1		QHD$K_SIZ+
	1		(%LOC(MID(I).MID$L_RNA)-%LOC(MID(1)))
	    FLINK=(FLINK_OFF/512)+1	! Record number
	    IF(FLINK .GT. MAXREC .OR. 
	1	IFSET(BITMAP,FLINK-1) .OR. 
	1	FLINK_OFF .LT. OFF_FRST_MSG)THEN
		TYPE *,' RNA pointer is corrupt, will skip RNA message'
		WRITE(*,87)FLINK,FLINK_OFF
87		FORMAT(' RNA FLINK BLOCK: ',Z8.8,' RNA FLINK_OFF: ',Z8.8)
		MID(I).MID$L_RNA=0	! Zero rna
		MID(I).MID$L_BRNA=-4
		MSGCNT=0	! No message found yet
		GO TO 100
	    ENDIF
	    CALL MARK(BITMAP,FLINK-1)  ! mark as allocated already
	    WRITE(*,87)FLINK,FLINK_OFF
	    READ(QUELUN'FLINK)BLOCK ! read first block of RNA message

C
C 	    Read subsequent blocks of the RNA message
c
	    IF(BLOCK.QS2$L_DLNK(1) .NE. 0)THEN ! subsequent data block thd.
		NEXT_DATA_OFF=BLOCK.QS2$L_DLNK(1)+  ! calc. relative offset
	1		FLINK_OFF+			! diff. for first block
	1		(%LOC(BLOCK.QS2$L_DLNK)-%LOC(BLOCK.QS2$L_FLNK))
		LAST_DATA_LINK=FLINK_OFF+ ! Calc. list head beginning
	1		(%LOC(BLOCK.QS2$L_DLNK)- ! so know when to end the trav.
	1		%LOC(BLOCK.QS2$L_FLNK))  ! of data blocks of the message
120		CONTINUE	! for subsequent data blocks (blocks 3 on)
		NEXT_BLOCK=(NEXT_DATA_OFF/512)+1 ! RECORD NUMBER
		WRITE(*,87)NEXT_BLOCK,NEXT_DATA_OFF
		IF(NEXT_BLOCK .GT. MAXREC .OR. 
	1		IFSET(BITMAP,NEXT_BLOCK-1) .OR.
	1		NEXT_DATA_OFF .LT. OFF_FRST_MSG)THEN
			TYPE *,' RNA thread is corrupt, will skip RNA message'
			MID(I).MID$L_RNA=0
			MID(I).MID$L_BRNA=-4
			CALL UNMARK(BITMAP)  ! Don't mark these as allocated
			GO TO 100
		ENDIF
		CALL MARK(BITMAP,NEXT_BLOCK-1)	! mark block as in use
		READ(QUELUN'NEXT_BLOCK)BLOCK ! read data block of RNA message
		NEXT_DATA_OFF=NEXT_DATA_OFF+BLOCK.QS2$L_FLNK ! forward link
		IF(NEXT_DATA_OFF .NE. LAST_DATA_LINK)GO TO 120
	     ENDIF ! SUBSEQUENT DATA BLOCKS FOR RNA MESSAGE
	   ENDIF	! END OF RNA TYPE MESSAGE
	   CALL ALLOCATE	! perm. mark all allocated blocks as used

C
C	   VERIFY THE NON-RNA MESSAGES
C
100	   CONTINUE
	   MID_LHD=((I-1)*MID$K_LEN)+QHD$K_SIZ ! List head address
					! save so know when we are at end
c
c	    Start reading the messages and building a buffer
c	    for later reinput into the queue
c
	    FLINK_OFF=MID(I).MID$L_FLK + 	! link is relative to self
	1		QHD$K_SIZ+		! add our own relative offset
	1		(%LOC(MID(I).MID$L_FLK)-%LOC(MID(1)))
c
D	    WRITE(*,300)MID(I).MID$L_FLK,FLINK_OFF,MID_LHD
D300	    FORMAT(' Link to first message: ',Z8.8,/,
D	1		' relative offset: ',Z8.8,/,
D	1		' Message LHD offset: ',Z8.8)
C
C	    Loop here scanning messages for this id
c
350	    CONTINUE
	    FLINK=(FLINK_OFF/512)+1	! Read the message block
D	    WRITE(*,360)FLINK_OFF,FLINK
D360	    FORMAT(' Reading first block of message, offset: ',Z8.8,/,
D	1	       ' Block number: ',I6)
	    IF(FLINK .LT. OFF_FRST_MSG .OR. 
	1	   IFSET(BITMAP, FLINK-1) .OR.
	1	   FLINK .GT. MAXREC)THEN
	  	 WRITE(*,375)MID(I).MID$C_MID,FLINK_OFF
375		 FORMAT(' Queue link error on message id: ',A16,/,
	1		' link offset: ',Z8.8,/,
	1		' Continuing with next message id, ',
	1		' remaining messages this id are lost')
		 IF(SAVLNK .EQ. 0)THEN
		   MID(I).MID$L_FLK=0	! Is the first message corrupt?
		   MID(I).MID$L_BLK=-4	
		 ELSE
		   MID(I).MID$L_BLK=	! calc. new back link
	1		SAVLNK-(MID_LHD+
	1		(%LOC(MID(I).MID$L_BLK)-%LOC(MID(I))))
		   FLINK=(SAVLNK/512)+1  ! read previous mesg. block
		   READ(QUELUN'FLINK)BLOCK ! so can update forward thread
		   BLOCK.QS2$L_FLNK=(%LOC(BLOCK.QS2$L_FLNK)-  ! point back
	1		%LOC(BLOCK)+SAVLNK)-		     ! to listhead
	1		(MID_LHD+(%LOC(MID(I).MID$L_FLK)-
	1		%LOC(MID(I))))
		   WRITE(QUELUN'FLINK)	! put back in queue
		 ENDIF
		 GO TO 9000
	     ENDIF
	     CALL MARK(BITMAP,FLINK-1)  ! Mark the first block as allocated
C
C 	     Check the first data block of message
c
	     READ(QUELUN'FLINK)BLOCK	! read first block of  message 
	     LAST_DATA_LINK=FLINK_OFF+ ! Calc. list head beginning
	1		(%LOC(BLOCK.QS2$L_DLNK)- ! so know when to end the trav.
	1		%LOC(BLOCK.QS2$L_FLNK))  ! of data blocks of the message
c
D	     WRITE(*,400)BLOCK.QS2$L_FLNK,
D	1		BLOCK.QS2$L_BLNK,
D	1		BLOCK.QS2$L_DLNK(1),
D	1		LAST_DATA_LINK
D400	     FORMAT(' forward next message link: ',Z8.8,/,
D	1		' backward link: ',Z8.8,/,
D	1		' forward message data link: ',Z8.8,
D	1		' Listhead for the msg data blocks: ',Z8.8)
c
c
	     NEXT_MSG_LINK=BLOCK.QS2$L_FLNK ! Save forward offset next 
						! message in this id's queue
c
c	     see if there are subsequent data message blocks to be
c	     dethreaded
c
	     IF(BLOCK.QS2$L_DLNK(1) .NE. 0)THEN  ! have at least one
		NEXT_DATA_OFF=BLOCK.QS2$L_DLNK(1)+FLINK_OFF+
	1	  	(%LOC(BLOCK.QS2$L_DLNK)- ! NOTE: Subsequent blocks
	1		 %LOC(BLOCK.QST$L_FLNK)) ! have only single thread.
		NEXT_BLOCK=(NEXT_DATA_OFF/512)+1 ! Relative block number
D		WRITE(*,600)NEXT_DATA_OFF,NEXT_BLOCK
D600		FORMAT(' Offset to second block of message: ',z8.8,/,
D	1		 ' block number: ',I6)
		IF(NEXT_BLOCK .LT. OFF_FRST_MSG .OR. 
	1	   IFSET(BITMAP,NEXT_BLOCK-1) .OR.
	1	   NEXT_BLOCK .GT. MAXREC)THEN
		  	WRITE(*,375)MID(I).MID$C_MID,NEXT_DATA_OFF
			 CALL UNMARK(BITMAP) ! These are no longer used
		    IF(SAVLNK .EQ. 0)THEN
		      MID(I).MID$L_FLK=0	! Is the first message corrupt?
		      MID(I).MID$L_BLK=-4	
		    ELSE
		      MID(I).MID$L_BLK=	! calc. new back link
	1		SAVLNK-(MID_LHD+
	1		(%LOC(MID(I).MID$L_BLK)-%LOC(MID(I))))
		      FLINK=(SAVLNK/512)+1  ! read previous mesg. block
		      READ(QUELUN'FLINK)BLOCK ! so can update forward thread
		      BLOCK.QS2$L_FLNK=(%LOC(BLOCK.QS2$L_FLNK)-  ! point back
	1		%LOC(BLOCK)+SAVLNK)-		     ! to listhead
	1		(MID_LHD+(%LOC(MID(I).MID$L_FLK)-
	1		%LOC(MID(I))))
		      WRITE(QUELUN'FLINK)	! put back in queue
		    ENDIF
		    GO TO 9000
		ENDIF
		CALL MARK(BITMAP,NEXT_BLOCK-1) ! Mark as allocated
		READ(QUELUN'NEXT_BLOCK)BLOCK  ! Got second block of data

C
C		loop here traversing the data blocks (blocks 2 on)
c
1200		CONTINUE
		NEXT_DATA_OFF=NEXT_DATA_OFF+  ! relative offset to next
	1		BLOCK.QST$L_FLNK	  ! data block
		IF(NEXT_DATA_OFF .NE. 	! equell to LHD in first block
	1		LAST_DATA_LINK )THEN  ! of message? if so done
		NEXT_BLOCK=(NEXT_DATA_OFF/512)+1 ! Record number
D		WRITE(*,1300)NEXT_DATA_OFF,NEXT_BLOCK
D1300		FORMAT(' Block 3 on offset: ',Z8.8,
D	1		   ' Block number: ',I6)
		IF(NEXT_BLOCK .LT. OFF_FRST_MSG .OR. 
	1	      IFSET(BITMAP,NEXT_BLOCK-1) .OR.
	1	      NEXT_BLOCK .GT. MAXREC)THEN
		  	WRITE(*,375)MID(I).MID$C_MID,NEXT_DATA_OFF
			CALL UNMARK(BITMAP) ! UNALLOCATE THOSE ALLOCATED SO FAR
		 IF(SAVLNK .EQ. 0)THEN
		   MID(I).MID$L_FLK=0	! Is the first message corrupt?
		   MID(I).MID$L_BLK=-4	
		 ELSE
		   MID(I).MID$L_BLK=	! calc. new back link
	1		SAVLNK-(MID_LHD+
	1		(%LOC(MID(I).MID$L_BLK)-%LOC(MID(I))))
		   FLINK=(SAVLNK/512)+1  ! read previous mesg. block
		   READ(QUELUN'FLINK)BLOCK ! so can update forward thread
		   BLOCK.QS2$L_FLNK=(%LOC(BLOCK.QS2$L_FLNK)-  ! point back
	1		%LOC(BLOCK)+SAVLNK)-		     ! to listhead
	1		(MID_LHD+(%LOC(MID(I).MID$L_FLK)-
	1		%LOC(MID(I))))
		   WRITE(QUELUN'FLINK)	! put back in queue
		 ENDIF
		 GO TO 9000
		ENDIF
		CALL MARK(BITMAP,NEXT_BLOCK-1) ! ALLOCATE BLOCK
		READ(QUELUN'NEXT_BLOCK)BLOCK
C
		GO TO 1200	! go get rest of the data blocks
	      ENDIF  ! end of block dethreading for blocks 3 on of mesg.
	    ENDIF    ! end of block for block 2 of a message
	    CALL ALLOCATE	! Mark blocks as allocate perm.
c
c	    Get the next message for this message id
c
	    MSGCNT=MSGCNT+1	! incrment number of messages found
	    SAVLNK=FLINK_OFF	! save this block in case next mesg. is corrupt
D	    WRITE(*,1400)NEXT_MSG_LINK,MID(I).MID$W_CNT
D1400	    FORMAT(' NEXT_MSG_LINK: ',Z8.8,' MID$W_CNT: ',Z4.4)
	    IF(NEXT_MSG_LINK .NE. 
	1	(MID_LHD+%LOC(MID(I).MID$L_FLK)-%LOC(MID(I))) .AND.
	1	MID(I).MID$W_CNT .GT. MSGCNT)THEN	! got another message?
D		  TYPE *,' Getting next message in this id s thread'
D		  TYPE *,' Number found so far: ',MSGCNT
		  FLINK_OFF=NEXT_MSG_LINK+FLINK_OFF 
		  GO TO 350			! Go get all of it's blocks
	    ENDIF
9000	    CONTINUE
	    IF(MID(I).MID$W_CNT .NE. MSGCNT)THEN
		TYPE *,' Message counts do not agreee for mid: '
		TYPE *,MID(I).MID$C_MID
		TYPE *,' Resetting to: ',MSGCNT
		MID(I).MID$W_CNT=MSGCNT
	    ENDIF
10000	CONTINUE
C
C Update the message id list head in the event that they
c have changed.
c
	K=0
	DO I=(QHD$K_SIZ/512)+1,BLOCK_COUNT+(QHD$K_SIZ/512)
		K=K+1
		WRITE(QUELUN'I)(BLOCKS(J,K),J=1,128)
	ENDDO
	RETURN
	END
	SUBROUTINE FREE_LIST(BITMAP,BITMAPSIZE,MAXREC,QHD)
	IMPLICIT NONE
C
C This subroutine is responsible for traversing the
c free list thread.  The following describes how this
c will be done:
c
c 1) zero out the bit map
C 2) traverse the free list thread and verify that
c    the thread points to a valid block.
c 3) mark the block as used by setting the bit in the
c    bitmap
c
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QHDDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(HDRDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QSTDEF)/LIST'
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	RECORD/QHDDEF/QHD	! QUEUE HEADER
	INTEGER*4 QUELUN	! QUEUE FILE LUN
	PARAMETER(QUELUN=1)
	INTEGER*4 BITMAPSIZE	! NUMBER OF LONG WORDS IN BITMAP
	INTEGER*4 BITMAP(BITMAPSIZE)	! BITMAP
	INTEGER*4 MAXREC	! LARGEST VALID BLOCK IN THE QUEUE
	INTEGER*4 I		! LOOP VARIABLE
	INTEGER*4 FREE_COUNT	! NUMBER OF FREE BLOCKS FOUND
	INTEGER*4 FBLOCK	! FORWARD BLOCK NUMBER
	INTEGER*4 BBLOCK	! BACKWARD BLOCK NUMBER
	INTEGER*4 FLINK		! FORWARD LINK 
	INTEGER*4 BLINK		! FORWARD LINK
	INTEGER*4 CUROFF	! CURRENT BLOCK
	INTEGER*4 LHDOFF	! OFFSET TO THE LIST HEAD
	INTEGER*4 PRVOFF	! PREVIOUS BLOCK (FOR TESTING BACK LINKS)
	RECORD/QSTDEF/BLOCK	! RECORD TO READ FREE BLOCKS INTO

c
D	TYPE *,' Verify the FREE BLOCK LIST'
	FREE_COUNT=0		! NUMBER OF BLOCKS FOUND ON THE FREE LIST
	LHDOFF= %LOC(QHD.QHD$Q_FLS(1))-%LOC(QHD) ! OFFSET FROM START OF QUEUE
	PRVOFF=LHDOFF		! PREVIOUS BLOCK
	DO I=1,BITMAPSIZE
		BITMAP(I)=0	! INIT BY ZEROING OUT THE BITMAP
	ENDDO
C
	FLINK=QHD.QHD$Q_FLS(1)+LHDOFF ! SEE WHERE FORWARD LINK POINTS TO
	BLINK=QHD.QHD$Q_FLS(2)+ ! SEE WHERE BACKWARD LINK POINTS TO
	1	(%LOC(QHD.QHD$Q_FLS(2))-%LOC(QHD)) ! OFFSET FROM START OF QUEUE
	IF(QHD.QHD$Q_FLS(1) .EQ. 0)THEN	! if zero, then no good
		TYPE *,' free list LIST HEAD is corrupt, resetting'
		QHD.QHD$Q_FLS(1)=LHDOFF	! RESET THE WHOLE THING
		QHD.QHD$Q_FLS(2)=-4
	ENDIF
C
C LOOP HERE TRAVERSING THE QUEUE
C	
50	CONTINUE
	IF(FLINK .EQ. LHDOFF)THEN	! ARE WE DONE?
	   TYPE *,' Free list threaded correctly'
	   TYPE *,' Number of free blocks: ',FREE_COUNT
	   IF(QHD.QHD$L_FREE .NE. FREE_COUNT) THEN
		TYPE *,' However, does not match queue header: ',
	1		QHD.QHD$L_FREE,' Ressetting'
		QHD.QHD$L_FREE=FREE_COUNT
	   ENDIF
	   RETURN
	ENDIF
C
D	WRITE(*,100)FLINK,BLINK,PRVOFF
D100	FORMAT(' FORWARD  LINK OFFSET: ',Z8.8,/,
D	1      ' BACKWARD LINK OFFSET: ',Z8.8,/,
D	1      ' PREVIOUS OFFSET:      ',Z8.8)
C
	FBLOCK=(FLINK/512)+1	! Forward block number
	BBLOCK=(BLINK/512)+1	! Backward block number
	IF(BBLOCK .LT. 1 .OR. BBLOCK .GT. MAXREC)THEN
		TYPE *,' Backward thread invalid'
		TYPE *,' BBLOCK: ',BBLOCK
		IF(FREE_COUNT .EQ. 0)THEN  ! LIST HEAD CORRUPT?
		  QHD.QHD$Q_FLS(1)=LHDOFF
		  QHD.QHD$Q_FLS(2)=-4	! INIT TO EMPTY LIST
		  QHD.QHD$L_FREE=0
		ELSE			! TERMINATE THE LIST HERE
		  TYPE *,' Terminating free list chain'
		ENDIF
	ENDIF
C
	IF(FBLOCK .LT. 1 .OR. FBLOCK .GT. MAXREC)THEN
		TYPE *,' Forward thread invalid'
		TYPE *,' FBLOCK: ',FBLOCK
		IF(FREE_COUNT .EQ. 0)THEN  ! LIST HEAD CORRUPT?
		  QHD.QHD$Q_FLS(1)=LHDOFF
		  QHD.QHD$Q_FLS(2)=-4	! INIT TO EMPTY LIST
		  QHD.QHD$L_FREE=0
		ELSE
		  TYPE *,' Terminating free list chain'
		ENDIF

	ENDIF
C
C READ THE NEXT BLOCK
C
	FREE_COUNT=FREE_COUNT+1 ! INCREMENT NUMBER OF FREE BLOCKS
	CALL SETBIT(BITMAP,FBLOCK-1)  ! SET BLOCK AS IN USE
	CUROFF=FLINK	! CURRRENT OFFSET
D	TYPE *,' Reading block number: ',FBLOCK
	READ (QUELUN' FBLOCK)BLOCK  ! GET NEXT BLOCK
	FLINK=BLOCK.QST$L_FLNK+CUROFF	! FORWARD LINK
	BLINK=BLOCK.QST$L_BLNK+CUROFF	! BACKWARD LINK
	IF(BLINK .NE. PRVOFF)THEN	! is the backward link valid
	   TYPE *,' Backward link corrupt'
	   WRITE(*,500)FLINK,BLINK,CUROFF,PRVOFF
500	   FORMAT(' FLINK: ',Z8.8,' BLINK: ',Z8.8,/,
	1	  ' CUROFF: ',Z8.8,' PRVOFF: ',Z8.8)
	   BLOCK.QST$L_FLNK=LHDOFF-CUROFF	! POINT TO LIST HEAD
	   BLOCK.QST$L_BLNK=PRVOFF	! POINT TO PREVIOUS BLOCK
	   WRITE(QUELUN ' FBLOCK)BLOCK	! UPDATE LAST BLOCK
	   QHD.QHD$L_FREE=FREE_COUNT  	! STORE CURRENT COUNT
	   QHD.QHD$Q_FLS(2)=CUROFF-(LHDOFF+4)	! BACK POINTS TO US
	   RETURN
	ENDIF
	PRVOFF=CUROFF	! Previous will point to old current
	GO TO 50	! GO VALIDATE THIS ONE
	END
	SUBROUTINE MARK(BITMAP,BIT)
	IMPLICIT NONE
C
C This subroutine has three entries.  The first is to mark a block
c as used (but remember in event that we have to reconsider it),
c The seconds is to unmark the ones we just marked, and the third
c entry is to forget those we were trying to remember.
c
	INTEGER*4 BITMAP(*)	! ARRAY OF BITS
	INTEGER*4 BIT		! BIT POSITON 0 TO # OF BITS IN ARRAY
	INTEGER*4 STACK(100)	! OUR MEMORY
	INTEGER*4 ISTACK	! CURRNET DEPTH OF STACK
	INTEGER*4 I		! LOOP VARIABLE
	DATA ISTACK/0/
C
C SET THE DESIRED BIT AND REMEMBER IT
C
D	TYPE *,' Marking block: ',BIT,' As allocated'
	CALL SETBIT(BITMAP,BIT)
	IF(ISTACK .GT. 100)RETURN
	ISTACK=ISTACK+1
	STACK(ISTACK)=BIT	! PUT AWAY FOR SAVE KEEPING
	RETURN
C
	ENTRY UNMARK(BITMAP)
C
C UNMARK ALL THOSE IN OUR MEMORY
C
	IF(ISTACK .EQ. 0)RETURN	! NO MEMORY?
	DO I=1,ISTACK
		CALL CLRBIT(BITMAP,STACK(I))
	ENDDO
	RETURN
C
	ENTRY ALLOCATE
C
C ERASE OUR MEMORY
C
	ISTACK=0
	RETURN
	END
	SUBROUTINE RECLAIM(QHD,BITMAP,START_BLOCK,END_BLOCK)
	IMPLICIT NONE
C
C This subroutine is responsible for scanning the bit map
c an recovering any unused blocks by placing them back onto
c the free list chain.  The method used is to simply scan
c the bit map for any bits that are clear and if so, place
c the associated block back into the chain
c
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QHDDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(HDRDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(QSTDEF)/LIST'
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	RECORD/QHDDEF/QHD	! QUEUE HEADER
	RECORD/QSTDEF/BLOCK	! messge blocks
	INTEGER*4 QUELUN	! QUEUE FILE LUN
	PARAMETER(QUELUN=1)
	INTEGER*4 BITMAP(*)	! BITMAP
	INTEGER*4 I
	INTEGER*4 RECOVER_COUNT
	INTEGER*4 START_BLOCK	! Starting block of messages blocks
	INTEGER*4 END_BLOCK	! Ending block of message blocks
	LOGICAL*4 IFSET		! Test status of a bit in bit array
	INTEGER*4 FLINK		! forward link block number
	INTEGER*4 FLINK_OFF	! forward link from list head

	RECOVER_COUNT=0
	DO 1000 I=START_BLOCK,END_BLOCK
	  IF(.NOT. IFSET(BITMAP,I-1))THEN
		RECOVER_COUNT=RECOVER_COUNT+1
		TYPE *,' BLOCK: ',I,' Is being recovered'
		FLINK_OFF=QHD.QHD$Q_FLS(2)+	! Get current end block free 
	1	  (%LOC(QHD.QHD$Q_FLS(2))-%LOC(QHD)) 
		IF (QHD.QHD$L_FREE .NE. 0)THEN  ! do we have to thread to last?
		  FLINK=(FLINK_OFF/512)+1		! block number to read
		  READ(QUELUN'FLINK)BLOCK  ! Read last in free list
		  BLOCK.QS2$L_FLNK= ((I-1)*512) - FLINK_OFF ! New forward link
		  WRITE(QUELUN'FLINK)BLOCK	!  Put new threaded block back
		  BLOCK.QS2$L_BLNK=(512*(I-FLINK))- ! Relative back ptr to 
	1	   (%LOC(BLOCK.QS2$L_FLNK)-%LOC(BLOCK.QS2$L_BLNK))  ! last blk
		ELSE
		  BLOCK.QS2$L_BLNK=(%LOC(QHD.QHD$Q_FLS(1))-
	1		%LOC(QHD))-((I-1)*512)   ! Backward back to LHD (only 
						 ! block on free list thread)
		ENDIF
		BLOCK.QS2$L_FLNK=(%LOC(QHD.QHD$Q_FLS(1))-
	1		%LOC(QHD))-((I-1)*512)  ! relative back link to LHD
		WRITE(QUELUN'I)BLOCK		! write new block
		QHD.QHD$L_FREE=QHD.QHD$L_FREE+1 ! Increment number free
	  ENDIF
1000	CONTINUE
C
	TYPE *,' Total number of blocks recovered: ',RECOVER_COUNT
	RETURN
	END
	SUBROUTINE CLRBIT(BITARY,IBIT)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&CLRBIT\&
C .nf
C .x CLRBIT>Defined
C Source:CLRBIT.MOD
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 3/26/87
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C 
C BITARY                        BYTE
C                               Array of bits
c 
c IBIT                          INTEGER*4
C                               Desired bit to clear within array.
C  
C Returns:
C 
C  
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine is responsible for clearing a bit within a bit
c array to one.  The calling sequence is:
c .nofill
c .sk
c 	INTEGER*4 BITARY(100)	! BIT ARRAY
C 	INTEGER*4 IBIT		! BIT TO SET STARTING WITH ZERO
C 	CALL CLRBIT(BITARY,IBIT)
C 
C end.doc ****************************** end.doc  

	BYTE BYTMSK(0:7)	! SINGLE BYTE MASK
	BYTE BITARY(0:*)	! BIT ARRAY FROM USER
	INTEGER*4 IBIT	! BIT TO TEST
	INTEGER*4 IR	! REMAINDER BIT
	INTEGER*4 IB	! BYTE BIT IS IN
C
	DATA BYTMSK/'1'X,'2'X,'4'X,'8'X,'10'X,'20'X,'40'X,'80'X/
C
C	
	IB=IBIT/8		! GET BYTE THAT CONTAINS BIT
	IR= IBIT- (IB*8)	! GET BIT WITHIN BYTE
C
	BITARY(IB)=BITARY(IB) .AND. (.NOT.(BYTMSK(IR))) ! SET THE BIT
	RETURN
	END
	SUBROUTINE SETBIT(BITARY,IBIT)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&SETBIT\&
C .nf
C .x SETBIT>Defined
C Source:SETBIT.MOD
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 3/26/87
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C 
C BITARY                        BYTE
C                               Array of bits
c 
c IBIT                          INTEGER*4
C                               Desired bit to set within array.
C  
C Returns:
C 
C  
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine is responsible for setting a bit within a bit
c array to one.  The calling sequence is:
c .nofill
c .sk
c 	INTEGER*4 BITARY(100)	! BIT ARRAY
C 	INTEGER*4 IBIT		! BIT TO SET STARTING WITH ZERO
C 	CALL SETBIT(BITARY,IBIT)
C 
C end.doc ****************************** end.doc  

	BYTE BYTMSK(0:7)	! SINGLE BYTE MASK
	BYTE BITARY(0:*)	! BIT ARRAY FROM USER
	INTEGER*4 IBIT	! BIT TO TEST
	INTEGER*4 IR	! REMAINDER BIT
	INTEGER*4 IB	! BYTE BIT IS IN
C
	DATA BYTMSK/'1'X,'2'X,'4'X,'8'X,'10'X,'20'X,'40'X,'80'X/
C
C	
	IB=IBIT/8		! GET BYTE THAT CONTAINS BIT
	IR= IBIT- (IB*8)	! GET BIT WITHIN BYTE
C
	BITARY(IB)=BITARY(IB) .OR. BYTMSK(IR) ! SET THE BIT
D	WRITE (*,100)IBIT,IB,IR,BITARY(IB)
D100	FORMAT(' bit: ',I4,' byte: ',I4,' bit position: ',I1,' BYTE: ',Z2.2)
	RETURN
	END
