	OPTIONS/EXTEND
	INTEGER*4 FUNCTION NEW_ROUTING_MAP(EVENT)
	IMPLICIT NONE
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&NEW_ROUTING_MAP\&
C .nf
C .x NEW_ROUTING_MAP>Defined
C Source:NEW_ROUTING_MAP.MOD
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 06-APR-87
C Revision level     :1.7
C  
C .C ;Formal Parameter List
C Receives:
C  
C Returns:
C  
C    NEW_ROUTING_MAP          INTEGER*4                                         
C                               RETURN STATUS                                     
C  
C Accesses common(s):
C  
C .X MA_ROUTER_COMMON>Common
C    MA_ROUTER_COMMON           PROCESS COMMON FOR CHANNELS, V.A., ETC.           
C  
C .X ROUTE_TABLES>Common
C    ROUTE_TABLES               IF PRESENT, CONTAINS THE CURRENT ROUTING          
C                               DATABASE                                          
C  
C Accesses file(s):
C  
C    ROUTE_DATABASE.EXE                                                           
C                               READ                                              
C .X ROUTE_DATABASE.EXE>READ
C  
C Other modules referenced:
C  
C .X OPR_FAO_MSG>Referenced
C    OPR_FAO_MSG                USED TO LOG MESSAGE TO OPERATOR CONSOLE           
C                                (found in library:INLAND)
c 
c .X AT_ADD>Referenced
c    AT_ADD                     Used to return a value at the given address
c                                (found in library: INLAND)
C
C .X RELOCATE_OFFSET
C    RELOCATE_OFFSET            Used to relocate link in the database
c                                 (found in library: INLAND)
C .X DISCONNECT_ALL>Referenced
c    DISCONNECT_ALL             DISCONNECT ALL LOGICAL LINKS
C  
C .X BUILD_ROUTE_TABLE>Referenced    
C    BUILD_ROUTE_TABLE          BUILD ROUTING TABLE ENTRY
c  
C 
C .X CONNECT_NODES>Referenced    
C    CONNECT_NODE	        CONNECT A REMOTE NODE
C
c  .X ATTACH_Q>Referenced
c     ATTACH_Q                  ATTACH TO THE QUEUE SECTION
C .X MAP_IDS>Referenced
c    MAP_IDS	                MAP IDS IN THE NRB
C
C .X GET_NODE_NAME>Referenced
c    GET_NODE_NAME              GET NODE NAME FROM REGION
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine is called to build a new routing table
C database.  The following actions are performed in order to do this:
C .list
C .le;Find the size of the new route table by openning the
C ROUTE_DATABASE.EXE file and getting the FAB$L_ALQ parameter.
C .le;Create and map the section with the new routing information
C .LE;Find out how many message ids will be serviced by us and
c request that the ATTACH_Q subroutine allocate sufficient
c storage for all of our message ids.
C .le;Scan the attached Queue Region for message ID's that
C are specified in the routing map and build and store
C a virtual address to the message ID listhead in the queue.  
C Mark any that are not found such that no messages will be received.
C .le;Initilize the ROUTE_DATABASE and save the mapped virtual
C address in our common.
C .els
C 	The calling sequence is:
C .nofill
C .sk
C    SYS_STATUS=NEW_ROUTING_MAP()
C 
C end.doc ****************************** end.doc  
C
C MODIFICATION HISTORY:
C	2/25/87	E. LAKIA (1.5)
C	Added support for testing the revision and flag word of
c	the routing database.
c
C	2/24/87  E. LAKIA(1.4)
C	Added the connection for the NET_STATUS message id.
c

	INCLUDE '($SSDEF)'
	INCLUDE '($IODEF)'
	INCLUDE 'MA_ROUTER_COMMON.INC/LIST'
	INCLUDE 'MA_ROUTER_MSG.INC'
	include 'MA_Q:[DEF]QUEUE.TLB(NRBDEF)/LIST'
	include 'MA_Q:[DEF]QUEUE.TLB(NETMSG)/LIST'
	INTEGER*4 SYS_STATUS	! system service status
	INTEGER*4 GET_NODE_NAME ! GET NODE NAME FROM REGION
	INTEGER*4 SYS$WAITFR	! WAIT FOR SINGLE EVENT FLAG
	INTEGER*4 SYS$BINTIM	! ASCII TO BINARY TIME CONVERSION
	INTEGER*4 SYS$SETIMR	! SET A TIMER
	INTEGER*4 SYS$CREPRC	! create process
	INTEGER*4 SYS$EXIT
	INTEGER*4 SYS$GETMSG	! GET SYSTEM MESSAGE
	INTEGER*4 SYS$DGBLSC	! DELETE GLOBAL SECTION
	INTEGER*4 SYS$DELTVA	! DELETE VIRTUAL ADDRESS SPACE
	INTEGER*4 SYS$DASSGN	! DEASSIGN CHANNEL
	INTEGER*4 SYS$SETAST	! AST DISABLE/ENABLE
	INTEGER*4 OPR_FAO_MSG	! OPERATOR LOG MESSAGE
	INTEGER*4 AT_ADD	! RETURN VALUE AT THE ADDRESS
	INTEGER*4 BUILD_ROUTE_TABLE ! BUILD ROUTING TABLE ENTRY
	INTEGER*4 CONNECT_NODE	! CONNECT A REMOTE NODE
	INTEGER*4 MAP_IDS	! MAP IDS IN THE NRB
	INTEGER*4 ATTACH_Q	! ATTACH TO QUEUE SECTION FILE
	INTEGER*4 GET_MID_INDEX	! GET message id index for NET_STATUS mesg. id
	INTEGER*4 EVENT		! EVENT THAT CAUSED US TO BE CALLED
				! NORMALLY, NEW DATABASE RECEIVED
	INTEGER*4 IOSTAT	! FORTRAN I/O STATUS
	EXTERNAL ROUTE_SECTION_OPEN ! SECTION OPEN SUBROUTINE
	INTEGER*4 I		! INDEX
	INTEGER*4 IOFF		! OFFSET VARIABLES USED IN ADDRESS CALC.
	INTEGER*2 MIDCNT	! NUMBER ID'S TO MAP
	INTEGER*4 RETRY_TIMER(2) ! 1 MINUTE TIMER

C
C READY TO CREATE NEW  REGION FILE/SECTION.
C
1000	CONTINUE
	PRILOG_COUNT=0	! set number of logical type node names to zero
C
	OPEN(  UNIT=ROUTE_DBS(5),
	1	NAME='ROUTE_DATABASE.DBS',
	1	TYPE='OLD',
	1	USEROPEN=ROUTE_SECTION_OPEN,
	1	ERR=2000,
	1	IOSTAT=IOSTAT)
C
D	SYS_STATUS=OPR_FAO_MSG(RTR_OPN,%VAL(ROUTE_DBS(3)))
C
C CALL TO BUILD AND RELOCATE THE JUST READ ROUTING MAP
c and connect to the global section queue.
c	The followging modules may have to be recompiled/assembled
c depending on the structure of the routing database header.
c	FIND_ROUTE_MSG.MAR
C	NEW_ROUTING_MAP.FOR
C	NODE_TO_NRB.FOR
C	UNFULL_AST.MAR
C	NRBDMP.FOR (NRB DUMP PROGRAM, NOT PART OF ROUTER)
C
C STRUCTURE OF ROUTING DATABASE HEADER IS:
C OFFSET
c  0	.long	version number
c  4	.long	flags
C  8	.LONG   OFFSET TO RESPONSE BUFFERS LIST
C 12	.LONG   OFFSET TO OUTPUT BUFFERS LIST
C 16	.LONG	NUMBER OF MESSAGE IDS THAT WILL BE ROUTED
C 	.REPT	NUMBER OF NRBS
C 20	.LONG 	OFFSET TO NRB (I)
C	.ENDR
C	.LONG	0
C
c
C
C ATTACH TO THE GLOBAL QUEUE SECTION FILE.
C WE WILL NOT PASS A QUEUE NAME BUT WILL USE WHAT
C EVER SOMEONE HAS SET UP AS THE QUEUE NAME.
C
C BEFORE USING, HOWEVER, CHECK FOR COMPATIBLE VERSIONS AND THAT
C THE REGION IS A VIRGIN REGION.
C
	IF(AT_ADD(%VAL(ROUTE_DBS(1))) .NE. VERSION)THEN
		TYPE *,' ROUTING DATABASE NOT AT CORRECT REVISON LEVEL'
		SYS_STATUS=OPR_FAO_MSG(RTR_INCOMPVERSION,
	1		%VAL(VERSION),
	1		%VAL(AT_ADD(%VAL(ROUTE_DBS(1)))))
		SYS_STATUS=SYS$EXIT(RTR_INCOMPVERSION)
	ENDIF
C
	IF(AT_ADD(%VAL(ROUTE_DBS(1)+4)) .NE. 0)THEN
		WRITE(*,20)AT_ADD(%VAL(ROUTE_DBS(1)+4))
20		FORMAT(' FLAG WORD: ',Z8.8)
		TYPE *,' ROUTING DATABASE IS NOT VIRGIN'
		SYS_STATUS=OPR_FAO_MSG(RTR_VIRGIN)
		SYS_STATUS=SYS$EXIT(RTR_VIRGIN)
	ENDIF
C
	CALL RELOCATE_OFFSET(%VAL(ROUTE_DBS(1)+4),ROUTE_DBS(1)) !
C			SET THE VIRGIN FLAG= ADDRESS (IE: 0 RELOCATED)
C
	MIDCNT=AT_ADD(%VAL(ROUTE_DBS(1)+16))
	TYPE *,' NUMBER OF IDS THAT MUST BE CONNECTED: '
	TYPE *,' IS: ',MIDCNT

	NEW_ROUTING_MAP=ATTACH_Q(Q_BLOCK ,	! ATTACH TO QUEUE
	1	,MIDCNT)
C
	IF(.NOT. NEW_ROUTING_MAP)THEN		! IF CAN'T ATTACH DON'T CONTINUE
		TYPE *,' UNABLE TO ATTACH TO QUEUE, FATAL EXIT'
		TYPE *,' STATUS: ',NEW_ROUTING_MAP
		MSGDESC(1)=512
		SYS_STATUS=SYS$GETMSG(%VAL(NEW_ROUTING_MAP),MSGDESC(1),
	1	MSGDESC,,)
		SYS_STATUS=OPR_FAO_MSG(RTR_QUEFAL,MSGDESC)
		SYS_STATUS=SYS$EXIT(%VAL(NEW_ROUTING_MAP))
	ELSE
		SYS_STATUS=GET_NODE_NAME()
		TYPE *,' OUR SOURCE NODE NAME IS: ',OUR_NODE
	ENDIF
C
	WRITE(*,700)Q_BLOCK
700	FORMAT(' QBLOCK: ',Z8,1X,Z8)
c
c Get the message id index for the net_status message id
c
	SYS_STATUS=GET_MID_INDEX(
	1	Q_BLOCK,
	1	NET_MSG_ID,
	1	NETMSGINDEX)  ! GET THE INDEX FOR EASY USE LATER
	IF(.NOT. SYS_STATUS)THEN
		MSGDESC(1)=512
		SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),MSGDESC(1),
	1	MSGDESC,,)
		SYS_STATUS=OPR_FAO_MSG(RTR_NETMSGID,MSGDESC)
	ENDIF

C
C RELOCATE THE RESPONSE BUFFERS AND THE OUTPUT
C BUFFERS
C
C	RESPONSE BUFFERS
C
	RESP_ADD_LST=AT_ADD(%VAL(ROUTE_DBS(1)+8))  ! ADDRESS OF RESP BUFFER
C						   ! LIST STRUCTURE
	WRITE(*,820)RESP_ADD_LST
820	FORMAT(' RESPONSE LIST OFFSET IN ROUTE DBS: ',Z8)
	RESP_COUNT=AT_ADD(%VAL(RESP_ADD_LST+
	1	ROUTE_DBS(1)))! GET COUNT FROM TOP OF LIST
	TYPE *,' NUMBER OF RESPONSE BUFFERS: ',RESP_COUNT
	RESP_INUSE=RESP_ADD_LST+4 +ROUTE_DBS(1)	! ADDRESS OF IN USE BYTE VECTOR
	RESP_ADD_LST=RESP_ADD_LST+(RESP_COUNT)+4 ! START OF ADDRESSES 
C						! SKIP COUNT
C
	RESP_ADD_LST=RESP_ADD_LST+ROUTE_DBS(1)	! RELOCATE ADDRESS
	WRITE(*,830)RESP_ADD_LST
830	FORMAT(' RESPONSE BUFFER ADDRESS LIST: ',Z8)
	DO I=0,RESP_COUNT-1	! RELOCATE BUFFER ADDRESSES FROM THE OFFSET
	   CALL RELOCATE_OFFSET(%VAL(RESP_ADD_LST+(I*4)),
	1	ROUTE_DBS(1)) ! RELOCATE ADDRESSES IN  TABLE OF RESP. BUFFERS
	ENDDO
	LAST_RESP=1	! SET TO FIRST
C
C	 RELOCATE THE OUTPUT BUFFERS
C
	BUF_ADD_LST=AT_ADD(%VAL(ROUTE_DBS(1)+12))  ! OFFSET OF OUTPUT BUFFER
C						   ! LIST STRUCTURE
	BUF_COUNT=AT_ADD(%VAL(BUF_ADD_LST+
	1	ROUTE_DBS(1)))! GET COUNT FROM TOP OF LIST
	BUF_AVAIL=BUF_COUNT	! SAVE NUMBER FOR ALLOATE/DEALLOCATE ROUTINES
C
	IF(BUF_COUNT .LT. 2)THEN
		TYPE *,' INSUFFICIENT NUMBER OF OUTPUT BUFFERS, EXIT'
	ENDIF
C
	TYPE *,' OUTPUT BUFFER COUNT: ',BUF_COUNT
	BUF_INUSE=BUF_ADD_LST+4	+ ROUTE_DBS(1)	! ADDRESS OF IN USE BYTE VECTOR
	BUF_ADD_LST=BUF_ADD_LST+(BUF_COUNT)+4+ ! START OF ADDRESSES 
	1	ROUTE_DBS(1)			!  SKIP THE COUNT
	WRITE (*,840)BUF_ADD_LST
840	FORMAT(' START OF OUTPUT BUFFER ADDRESS LIST: ',Z8)
C
C GET SIZE OF OUTPUT BUFFERS
C
	BUF_SIZE= AT_ADD(%VAL(BUF_ADD_LST+4))-AT_ADD(%VAL(BUF_ADD_LST))
	TYPE *,' SIZE OF OUTPUT BUFFERS: ',BUF_SIZE
C
	DO I=0,BUF_COUNT-1	! RELOCATE BUFFER ADDRESSES FROM THE OFFSET
	   CALL RELOCATE_OFFSET(%VAL(BUF_ADD_LST+(I*4)),
	1		ROUTE_DBS(1)) ! RELOCATE ADDRESSES IN TABLE OF BUF. ADDR.
	ENDDO
	LAST_BUF=1	! SET TO FIRST

C
C RELOCATE THE NRB POINTER (OFFSETS)
C
	I=0
	NRB_COUNT=0	! NUMBER OF NRB'S DEFINED
	LAST_NRB=ROUTE_DBS(1)+20	! INIT LAST SEARCHED NRB TO FIRST ONE
1100	CONTINUE
	IOFF=AT_ADD(%VAL(ROUTE_DBS(1)+(I*4)+20)) ! GET OFFSET THIS ENTRY
	TYPE *,' NEW_ROUTING_MAP, NRB OFFSET: ',IOFF
	IF (IOFF .EQ. 0) GO TO 1200		! DONE?
	NRB_COUNT=NRB_COUNT+1			! NO, INCREMENT NRB COUNT
	NEW_ROUTING_MAP=BUILD_ROUTE_TABLE(%VAL(IOFF+ROUTE_DBS(1))) ! PASS NRB
	IF(.NOT. NEW_ROUTING_MAP) THEN
		SYS_STATUS=SYS$SETAST(%VAL(1))	! ENABLE AST'S
		RETURN
	ENDIF
C
C CALL TO HAVE THE MESSAGE ID'S FOR THIS NODE MAPPED TO QUEUE
C
	NEW_ROUTING_MAP=MAP_IDS(
	1	%VAL(ROUTE_DBS(1)+IOFF),	    ! NRB
	1	%VAL(ROUTE_DBS(1)+IOFF+NRB$L_SIZ))  ! ID LIST
	IF(.NOT. NEW_ROUTING_MAP)THEN
		SYS_STATUS=SYS$SETAST(%VAL(1))	! ENABLE AST'S
		RETURN
	ENDIF

C
C CALL TO HAVE NODE CONNECTION ATTEMPTED
C
c
c If there were logical type node names, then request the
c find service routine and wait one minute before attempting
c
	IF(PRILOG_COUNT .NE. 0)THEN	! were there any logical node names?
	   SYS_STATUS=SYS$CREPRC(,		! PID
	1			IMAGE_NAME,	! REQUEST FIND SERVICE
	1			SYSINPUT,	! PROCESS SYS$INPUT
	1			SYSOUTPUT,	! PROCESS SYS$OUTPUT
	1			SYSERROR,	! PROCESS SYS$ERROR
	1			,,	! PRIV, QUOTA
	1			PROC_NAME,
	1			,,,)	! BASE PRI, UIC, MBX UNIT, STATUS
	   IF(.NOT. SYS_STATUS .AND.	! bad status?
	1	SYS_STATUS .NE. SS$_DUPLNAM)THEN  ! ( duplicate creation ok)
		    MSGDESC(1)=512
	 	    SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	    MSGDESC(1),MSGDESC,,)
	  	    SYS_STATUS=OPR_FAO_MSG(RTR_CREPRC,MSGDESC)
	   ENDIF
C
C	   START A SHORT TIMER SO WILL CONNECT LATER AFTER FIND 
C	   SERVICE COMPLETES.
C
	   SYS_STATUS = SYS$BINTIM('0 :01:00',
	1			RETRY_TIMER)
	   IF(.NOT. SYS_STATUS)THEN
		  MSGDESC(1)=512
		  SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	     MSGDESC(1),MSGDESC,,)
	   	  SYS_STATUS=OPR_FAO_MSG(RTR_RTYTIM,MSGDESC)
		  SYS_STATUS=SYS$EXIT(%VAL(SYS_STATUS))
	    ENDIF
C
C	    ACTUALLY START THE TIMER
C
	    SYS_STATUS = SYS$SETIMR(, ! EVENT FLAG  (USER ZERO)
	1		%REF(RETRY_TIMER),	! TIMER VALUE
	1		, 		! AST ADDRESS
	1		)		 ! REQUEST ID (= NRB ADDRESS)
	    IF(.NOT. SYS_STATUS)THEN
		  MSGDESC(1)=512
		  SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	     MSGDESC(1),MSGDESC,,)
	   	  SYS_STATUS=OPR_FAO_MSG(RTR_RTYTIM,MSGDESC)
		  SYS_STATUS=SYS$EXIT(%VAL(SYS_STATUS))  ! FATAL BUG ERROR
	    ENDIF
C
C 	    WAIT WHILE FIND SERVICE FILLS IN THE NODE NAMES
C
	    SYS_STATUS=SYS$WAITFR(%VAL(0))
	ENDIF	  
	NEW_ROUTING_MAP=CONNECT_NODE(%VAL(ROUTE_DBS(1)+IOFF))
	IF(.NOT. NEW_ROUTING_MAP) THEN
		SYS_STATUS=SYS$SETAST(%VAL(1))	! ENABLE AST'S
		RETURN
	ENDIF
	I=I+1
	GO TO 1100
1200	CONTINUE
	TYPE *,' NUMBER OF NRBs FOUND: ',NRB_COUNT
C
C ALL DONE, RENABLE AST'S SO THE CONNECTION ACCPECTS AND THE LIKE CAN
C BE PROCESSED
C
	NEW_ROUTING_MAP=SS$_NORMAL
	SYS_STATUS=SYS$SETAST(%VAL(1))	! ENABLE AST'S
	RETURN

C
C UNABLE TO OPEN THE FILE
C
2000	TYPE *,' UNABLE TO OPEN FILE, STATUS: ',IOSTAT
	SYS_STATUS=SYS$GETMSG(%VAL(ROUTE_DBS(8)),MSGDESC(1),
	1	MSGDESC,,)
	SYS_STATUS=OPR_FAO_MSG(RTR_BOPN,MSGDESC)
	NEW_ROUTING_MAP=%LOC(RTR_BOPN)
	SYS_STATUS=SYS$SETAST(%VAL(1))	! ENABLE AST'S
	RETURN
	END
