C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&ADD_TIMES_VMS\&
C .nf
C .x ADD_TIMES_VMS>Defined
C Source:ADD_TIMES_VMS1.MOD
C TO BECOME PART OF LIBRARY:[2BOF.PROD.LIBRARIES]USER
C Designer           :ROBERT F. READ
C Author             :
C Date of last update:26-JUN-1985 10:00:37.19
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C  
C    ABS_TIME                   CHARACTER*23                            
C                               ABSOLUTE TIME - TO WHICH DELTA TIME IS TO BE ADDED 
C                               TIME IS IN STANDARD VAX/VMS TIME FORMAT -
C                               'dd-mmm-yyyy hh:mm:ss.cc'
C  
C    DELTA_TIME                 CHARACTER*16                                  
C                               DELTA TIME - TO BE ADDED TO ABSOLUTE TIME
C                               TIME IS IN STANDARD VAX/VMS FORMAT FOR DELTA TIME
C                               'dddd hh:mm:ss.cc'
C  
C Returns:
C  
C    TIME_TOT                   CHARACTER*23                                      
C                               SUMMATION OF ABS_TIME AND DELTA_TIME                    
C                               - IN STANDARD VAX/VMS FORMAT                      
C 
C    STATUS                     INTEGER*4 
C                               SS$_NORMAL INDICATES SUCCESSFUL COMPLETION;
C                               ANY STATUS OTHER THAN SS$_NORMAL SHOULD BE
C                               REGARDED AS AN ERROR CONDITION
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X SYS$BINTIM>Referenced
C    SYS$BINTIM                 CONVERTS TIME STRING TO INTERNAL BINARY FORMAT    
C                                (found in library:SYSTEM SERVICES)
C  
C .X SYS$ASCTIM>Referenced
C    SYS$ASCTIM                 CONVERT INTERNAL FORMAT TIME TO STANDARD VAX/VMS  
C                               (ASCII) FORMAT                                    
C                                (found in library:SYSTEM SERVICES)
C  
C .X LIB$SUBX>Referenced
C    LIB$SUBX                   ADDS TWO QUADWORD INTEGER VALUES                  
C                                (found in library:RUN-TIME)
C 
C .X SYS$USR_OPR_LOG>Referenced
C    SYS$USR_OPR_LOG            LOGS APPROPRIATE ERROR MESSAGE TO OPERATOR CONSOLE
C                                (found in library:USER)
C .SK
C .fill
C .SK
C Description:
C .sk    
C 
CUSAGE:  STATUS = ADD_TIMES_VMS ( ABS_TIME, DELTA_TIME, TIME_TOT )
C 
C 	This function performs a summation of two VAX/VMS time strings - 
C one of standard 23-character format and one of standard 16-character
C delta time format.  In order to perform the summation the time
C strings must first be converted to internal binary format, then added
C as quadword values, and finally the result is converted back to
C standard VAX/VMS format.  If an error condition is encountered during
C any of the conversions or during the summation, an error message is 
C logged to the operators console and an error status is returned to
C the calling program.  When an error condition is encountered process
C control is returned to the calling program immediately. 
C end.doc ****************************** end.doc  
C
C  ADD_TIMES_VMS - PERFORMS A SUMMATION OF TWO VMS TIME STRINGS
C
C
	INTEGER*4 FUNCTION ADD_TIMES_VMS ( ABS_TIME, DELTA_TIME, TIME_TOT )
	   IMPLICIT NONE
	   CHARACTER*23	ABS_TIME	! INPUT TIME IN VAX/VMS FORMAT
	   CHARACTER*16	DELTA_TIME	! INPUT TIME IN VAX/VMS DELTA FORMAT
	   CHARACTER*23	TIME_TOT	! RETURNED TIME IN VAX/VMS FORMAT

	! Local variables
	INTEGER*4 	ABS_TIME_B(2)	! Time 1 - internal format
	INTEGER*4 	DELTA_TIME_B(2)	! Time 2 - internal format
	INTEGER*4 	TIME_TOT_B(2)	! Time total - internal format

	! Integer functions
	INTEGER*4 	SYS$BINTIM	! System Service to convert time to
					!    internal format
	INTEGER*4 	SYS$ASCTIM	! System Service to convert time to
					!    VAX/VMS (ASCII) format
	INTEGER*4 	LIB$SUBX	! Run-time function to subtract two 
					!    quadwords

	! Define error messages
	CHARACTER*(*)	BINTIM_ERR_MSG	
	PARAMETER ( BINTIM_ERR_MSG =
	2    '%ADD_TIMES_VMS:  Binary time conversion error.' )
	CHARACTER*(*)	ASCTIM_ERR_MSG
	PARAMETER ( ASCTIM_ERR_MSG =
	2    '%ADD_TIMES_VMS:  Ascii time conversion error.' )
	CHARACTER*(*)	SUBX_ERR_MSG
	PARAMETER ( SUBX_ERR_MSG =
	2    '%ADD_TIMES_VMS:  Error adding quadwords.' )


C
C	Convert time strings to internal format
C
	ADD_TIMES_VMS = SYS$BINTIM ( ABS_TIME, ABS_TIME_B )
	IF ( .NOT. ADD_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( ADD_TIMES_VMS, BINTIM_ERR_MSG, 
	2					LEN(BINTIM_ERR_MSG) )
	   RETURN
	END IF

	ADD_TIMES_VMS = SYS$BINTIM ( DELTA_TIME, DELTA_TIME_B )
	IF ( .NOT. ADD_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( ADD_TIMES_VMS, BINTIM_ERR_MSG, 
	2					LEN(BINTIM_ERR_MSG) )
	   RETURN
	END IF

C
C	Add times - because delta time is stored as a negative value
C	   the summation is accomplished by subtracting the negative
C	   delta time from the absolute time
C
	ADD_TIMES_VMS = LIB$SUBX ( ABS_TIME_B, DELTA_TIME_B, TIME_TOT_B )
	IF ( .NOT. ADD_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( ADD_TIMES_VMS, SUBX_ERR_MSG, 
	2					LEN(SUBX_ERR_MSG) )
	   RETURN
	END IF

C
C	Convert time total to standard format
C
	ADD_TIMES_VMS = SYS$ASCTIM ( , TIME_TOT, TIME_TOT_B, )
	IF ( .NOT. ADD_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( ADD_TIMES_VMS, ASCTIM_ERR_MSG, 
	2					LEN(ASCTIM_ERR_MSG) )
	   RETURN
	END IF


	RETURN
	END
