	INTEGER*4 FUNCTION MAKE_SECTION(LUN,SIZE,REGION_NAME,SECT_ADDR)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&MAKE_SECTION\&
C .nf
C .x MAKE_SECTION>Defined
C Source:MAKE_SECTION1.MOD
C TO BECOME PART OF LIBRARY:PRODUCTION
C Designer           :EARL LAKIA
C Author             :
C Date of last update:10-OCT-1985 08:46:38.22
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C  
C    LUN                        INTEGER*4                                         
C                               LOGICAL UNIT NUMBER OF REGION FILE. MUST BE       
C                               IN THE RANGE OF 1 TO 10.                          
C  
C    SIZE                       INTEGER*4                                         
C                               SIZE OF THE REGION TO BE CREATED IN               
C                               512 BYTE BLOCKS.                                  
C  
C    REGION_NAME                CHARACTER*15                                      
C                               NAME OF THE REGION TO BE CREATED. THIS NAME       
C                               IS ALSO USED TO OPEN THE SECTION FILE ON          
C                               DISK BY APPENDING THE EXTENSION .SCT.             
C                               THE CURRENT DEFAULTS ARE USED FOR OPENNING THE    
C                               DISK SECTION FILE, THEREFORE, MAKE SURE THEY      
C                               ARE APPROIATE FOR THIS PROCESS. OTHERWISE         
C                               MULTIPLE SECTION FILES WILL BE CREATED.           
C  
C Returns:
C  
C    SECT_ADDR                  INTEGER*4  (2)
C                               THIS IS THE VIRTUAL ADDRESS OF THE 
C                               CREATED SECTION. IT WILL BE NEEDED
C                               BY ANYONE WISHING TO ACCESS THE
C                               SECTION.
C                               THE SECOND LONG WORD CONTAINS THE 
C                               LENGTH OF THE SECTION WHICH WILL
C                               BE NEEDED BY ANYONE WHO WISHES
C                               TO FORCE AN UPDATE OF THE SECTION
C                               TO DISK.
C  
C Accesses common(s):
C  
C .X SECT_CHAN>Common
C    SECT_CHAN                  THIS COMMON IS USED TO STORE THE CHANNEL NUMBER
C                               RELATED TO THE LOGICAL UNIT NUMBER. (UNIT 
C                               NUMBER IS USED AS AN INDEX)                    
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X GET_CHAN>Referenced
C    GET_CHAN                   THIS SUBROUTINE RETURNS A CHANNEL NUMBER TO THE   
C                               SECT_CHAN COMMON INDEXED BY THE LOGICAL UNIT 
C                               NUMBER OF THE OPEN. THIS SUBROUTINE IS CALLED     
C                               VIA A FORTRAN USER OPEN.                          
C                                (found in library:PRODUCTION)
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine allows a calling program to create a 
C mapped group global section.  The virtual address space
C selected is from the first available from the user process.
C The user should consult the system service manual
C for documentation on the $CRMPSC to fully understand this module.
C 	Possible errors that can occur are:
C .list
C .le;Not enough virtual address space available to map section
C .le;Logical unit number not in range
C .le;Insufficient privelge
C .le;Insufficient global pages
C .els
C .nofill
C CALLING SEQUENCE
C  
C     INTEGER*4 SYS_STATUS
C     INTEGER*4 LUN/1/
C     CHARACTER*15 REGION_NAME/'TEST_REGION'/
C     INTEGER*4 SIZE/100/
C     INTEGER*4 SECT_ADDR(2)
C C
C     SYS_STATUS=MAKE_SECTION(LUN,SIZE,REGION_NAME,SECT_ADDR)
C C
C     IF(.NOT.SYS_STATUS) THEN    ! UNABLE TO CREATE REGION
C                                 ! NOTE:  MAY NOT BE SS$_NORMAL!
C                                 ! SUCH AS SECTION ALREADY EXISTS.
C     ENDIF
C .FILL
C NOTE:  The section file is created as follows:
c [2BOF.PROD.DATA]<--REGION NAME-->.SEC.  For the
c above example, it would have been: 
c [2BOF.PROD.DATA]TEST_REGION.SEC;1.
c  
C end.doc ****************************** end.doc  

	IMPLICIT NONE
	INCLUDE '($SSDEF)'	! SYSTEM SERVICES DEFINITIONS
	INCLUDE '($SECDEF)'	! SECTIONS DEFINITIONS
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	INTEGER*4 SYS$CRMPSC	! CREATE AND MAP GLOBAL SECTION
	INTEGER*4	LUN	! LOGICAL UNIT NUMBER FOR SECTION FILE
	INTEGER*4 	SIZE	! SIZE OF SECTION IN BLOCKS
	CHARACTER*(*)	REGION_NAME ! NAME OF REGION
	CHARACTER*50	REGION_NAME_FILE ! NAME OF REGION NAME FILE
	INTEGER*4 	SECT_ADDR(2)	 ! CREATED SECTION VIRTUAL ADDRESS
					 ! RETURNED TO CALLER
	INTEGER*4 VIRT_ADDR(2)	! VIRT ADDRESS OF SECTION
	INTEGER*4 CHANNELS	! ARRAY CONTAINING CHANNEL NUMBERS FOR
				! SECTION FILES
	COMMON/SECT_CHAN/CHANNELS(10)	! ALLOW UNIT NUMBERS UP TO 10
	EXTERNAL SECT_CHAN	! SECTION CHANNEL USER OPEN ROUTINE
	INTEGER*4 SEC_FLAGS	! SECTION FLAGS
	INTEGER*4 SYS_ADR(2)	! VIRTUAL ADDRESS OF SECTION
	INTEGER SLTH,ILTH	! LENGTH OF STRINGS
	INTEGER I		! LOOP VARIABLE
	INTEGER ITI/11/		! SYS$INPUT FOR DEBUG
	INTEGER*4 IOST		! I/O STATUS FROM OPEN OF SECTION FILE

C

C
C VALIDATE THE USER INPUTS
C
	IF(LUN.LE.0 .OR. LUN .GT. 10)THEN
		MAKE_SECTION=SS$_IVCHAN	  ! RETURN INVALID CHANNEL ERROR CODE
		RETURN
	ENDIF
C
C TRUNCATE THE GLOBAL SECTION NAME AND CREATE THE SECTION FILE
C  
	DO 100 I=LEN(REGION_NAME),1,-1
		IF(REGION_NAME(I:I).NE.' ')THEN	! FOUND END OF STRING
C
C			BUILD THE FILE NAME STRING BY THE FOLLOWING
C			"[2BOF.PROD.DATA]<-REGION_NAME->.SEC
C
			REGION_NAME_FILE='[2BOF.PROD.DATA]'//
	1			REGION_NAME(1:I)//'.SEC;1'
			SLTH=I		! LENGTH OF SECTION
			ILTH=I+23	! LENGTH OF FILENAME STRING
			TYPE *,' REGION NAME FILE: ',
	1			REGION_NAME_FILE(1:ILTH)
			GO TO 200
		ENDIF
100	CONTINUE
200	CONTINUE
	OPEN(NAME='SYS$OUTPUT',TYPE='OLD',UNIT=ITI)
C
	OPEN(UNIT=LUN,NAME=REGION_NAME_FILE(1:ILTH),
	1	TYPE='UNKNOWN',ACCESS='DIRECT',
	2	MAXREC=SIZE,RECORDSIZE=128,
	3	USEROPEN=SECT_CHAN,
	4	INITIALSIZE=SIZE,
	5	IOSTAT=IOST,
	6	ERR=9000,
	7	SHARED)
C
	TYPE *,' OPEN COMPLETE'
	WRITE(ITI,150)CHANNELS(LUN)
150	FORMAT(' CHANNELS: ',Z10)

C
C CREATE THE SECTION, HAVE THE SYSTEM MAP IT WHERE
C EVER IT CAN AND GET ALL REQUIRED VIRTUAL 
C ADDRESS.  THIS SECTION IS GROUP GLOBAL AND
C WILL NOT EXITS IF NO PROCESS IS ATTACHED TO IT
C
	SEC_FLAGS=SEC$M_GBL		! GLOBAL SECTION FLAG (REQUIRED)
	1	.OR.SEC$M_WRT 		! READ/WRITE ACCESS
	2	.OR.SEC$M_EXPREG	! MAP WHEREEVER POSSIBLE
C
C
	VIRT_ADDR(1)=0	! LET EXEC GET MAPPING (IN P0 SPACE)
	VIRT_ADDR(2)=0	! MUST ZERO, INCASE CALLED MORE THAN ONCE
C
	SYS_STATUS=SYS$CRMPSC(VIRT_ADDR, ! STARTING VIRT ADDR. TO MAP SECTION
	1	SYS_ADR,		! ACTUAL MAPPED VIRT ADDR.
	2	%VAL(3),		! ACCESS MODE (USER)
	3	%VAL(SEC_FLAGS),	! SECTION FLAGS
	4	REGION_NAME(1:SLTH)	! SECTION NAME
	5	, , ,			! IDENT AND RELATIVE PAGE
	6	%VAL(CHANNELS(LUN)),  ! CHANNEL FOR SECTION FILE
	7	%VAL(SIZE)		! SIZE OF REGION
	8	, , , )	! START MAPPING OF VBN TO REGION, PROTECT, PFC
C
	TYPE *,' CREATE STATUS: ',SYS_STATUS
	MAKE_SECTION=SYS_STATUS  ! COPY STATUS FOR USER GOOD OR BAD
	SECT_ADDR(1)=SYS_ADR(1)	! RETURN REGION VIRTUAL ADDRESS TO CALLER	
	SECT_ADDR(2)=SYS_ADR(2) ! LENGTH OF REGION
C
	WRITE(ITI,700)VIRT_ADDR,SYS_ADR
700	FORMAT(' VIRT ADDRESS AND SYSTEM ADDRESS',/,4(1X,Z10))
	RETURN
C
C FILE OPEN ERROR ON SECTION FILE
C
9000	TYPE *,' FILE OPEN ERROR, STATUS:  ',IOST
	CALL ERRSNS(,SYS_STATUS,,,)
	TYPE *,' RMS STATUS: ',SYS_STATUS
	MAKE_SECTION=SS$_BADFILENAME
	RETURN
	END
