/* FIND, written by Joe Meadows, released to the world Sep. 8th, 1987.
   Note, this was originally called INDEX when posted to the internet,
   I changed the name to FIND to avoid confusion with the DECUS program
   INDEX which is a Fortran cross referencing utility.
   This is a public domain utility. No parts of this utility are
   copyrighted in any way. It may be used in any way, by anyone.
   There are no restrictions on this code. You may include it
   in a commercial package if you wish.

  Joe Meadows Jr.
  VAX/VMS System Manager / guru in training
  Fred Hutchinson Cancer Research Center
  1124 Columbia St.
  Seattle Wa. 98104
     bitnet - JOE@FHCRCVAX
     arpa   - JOE%FHCRCVAX.BITNET@WISCVM.WISC.EDU
     voice  - (206) 467-4970
*/

/* Modification History
   --------------------
   Sep.  8, 1987 JEM - Conversion from BETA to Release
   Sep. 25, 1987 JEM - modify all FID declaratiosn to unsigned
   Dec. 16, 1987 JEM - Changed name from INDEX to FIND, to avoid confusion
                       with another popular DECUS program
*/
#include stdio
#include "hm2def.h"
#include "fh2def.h"
#include "fm2def.h"
#include "fi2def.h"
#include "FileInfo.h"  /* includes FAB, NAM, and XAB */
#include "searchdef.h" /* includes DESCRIP */
#include "displaydef.h"

main()
{
  static struct dsc$descriptor device = {0, DSC$K_DTYPE_T, DSC$K_CLASS_D, 0};
  static struct dsc$descriptor devdesc = {0, DSC$K_DTYPE_T, DSC$K_CLASS_S, 0};
  static $DESCRIPTOR(defaultname,"");
  static $DESCRIPTOR(null,"\0");
  static unsigned short int indexf_fid[3] = {1,1,0};
  static unsigned short int fid[3] = {0,0,0};
  static struct dsc$descriptor filename = {0, DSC$K_DTYPE_T, DSC$K_CLASS_D, 0};
  static struct dsc$descriptor dynstr = {0, DSC$K_DTYPE_T, DSC$K_CLASS_D, 0};
  static long int pagecount=0,opagecount=0;
  static long int offset=1;
  static long int flag=0;
  static unsigned long lastfid=0,currfid=0,endfid=0;
  static struct FileInfo *context;
  static struct FAB *fabptr;
  static struct NAM *namptr;
  static struct display_info display;
  static struct search_info search;
  static struct hm2def *hm2;
  static struct fh2def *fh2;
  static struct fm2def *fm2;
  static long int fm2offset;
  static long int count,lbn;
  static struct fi2def *fi2;
  static long int status=1;
  static char *block=0;
  static long int blocknum;
  static short int chan=0,tmpchan=0;
  static long int i=0,j=0,k=0,volume=0;

  if (cli_present("OUTPUT") & 1) {
    /* we should see if they want output to go to a file or to SYS$OUTPUT */
    status = cli_get_value("OUTPUT",&dynstr);
    if (status & 1) {
      str$append(&dynstr,&null);
      freopen(dynstr.dsc$a_pointer,"w",stdout,"rfm=var","rat=cr");
    };
  };

  (void) init_display(&display);
  status = init_search(&search);
  if (!(status & 1)) return(status);

  if (cli_present("FULLNAME")  & 1) {
    /* if FULLNAME specified, use lib$find_file to get the file ids,
       get the header, and display */
    while (cli_get_value("FULLNAME",&dynstr) & 1) {
      i = 2; /* flag for lib$find_file, MULTIPLE bit set */
      while (lib$find_file(&dynstr,&filename,&fabptr,0,0,0,&i) & 1) {
        namptr = fabptr->fab$l_nam;
        /* assign a temporary channel to the device */
        devdesc.dsc$w_length = namptr->nam$t_dvi[0];
        devdesc.dsc$a_pointer = &(*namptr).nam$t_dvi[1];

        /* assign a channel to the device */
        status=sys$assign(&devdesc,&tmpchan,0,0);
        if (!(status & 1)) return(status);

        fh2 = get_header((*namptr).nam$w_fid,(*namptr).nam$w_fid,
                         context,1,tmpchan);
        if (fh2)
          if (search_header(&search,fh2,context,1,tmpchan))
            display_header(&display,namptr->nam$w_fid[0],fh2,
                           context,1,tmpchan);

        status=sys$dassgn(tmpchan);
      }; /* while (lib$find_file()) */
    }; /* while (cli_get_value("FULLNAME")) */
    /* if they did a /FULLNAME search, and didn't do a /FILE_ID
       search, then we could just quit now.. */
    if ((cli_present("FILE_ID") & 1) == 0)
      return;
  }; /* if (cli_present("fullname")) */

  /* get the name of the device which we're planning on checking */
  status=cli_get_value("P1",&device);
  if (!(status & 1)) return(status);

  /* add in logic for /VOLUME for volume sets. If set, then go through
     each of the index files, one at a time.. */
  if (cli_present("VOLUME") & 1) {
    /* Oh boy, we need to loop through all of the index files! */
    volume = 1;
    status = get_root_name(&device,&device);
    if (!(status & 1)) return(status);
  };

  /* loop through all the index files, this is accomplished via a call */
  /* to get_next_device at the end of the while loop (only if /VOLUME set) */
  while (status & 1) {
    /* assign a channel to the device */
    status=sys$assign(&device,&chan,0,0);
    if (!(status & 1)) return(status);

    /* find out the complete name of INDEXF.SYS (this is to be sure and get
       the device name right, even if you're sitting in a rooted directory */
    status = fid_to_name(&chan,indexf_fid,&filename);
    if (!(status & 1)) return(status);

    /* don't ask for more pages than we can handle, in fact, lets leave
       a mild safety net */
    pagecount = freptecnt() - 1000;

    /* if FILE_ID search, then map in a small part of the file */
    if (cli_present("FILE_ID") & 1)
      pagecount = 10;

    offset = 1;
    /* map in part of the file now */
    status=map_section(&filename,&defaultname,&pagecount,
                       &offset,&flag,&context);
    if (!(status & 1)) return(status);

    /* home block is usually block number 2 */
    hm2 = (struct hm2def *)(context->fi_l_begadr+512);

    if (hm2->hm2$b_struclev != 2) {
      printf("Sorry, FIND only works for structure level 2 disks.\n");
      return;
    };

    offset = hm2->hm2$w_ibmapvbn + hm2->hm2$w_ibmapsize;
    context->fi_l_recoff = offset - 1; /* Store the indexf offset here */
    opagecount = pagecount;
    pagecount -= (offset - 1);
    endfid = pagecount;
    lastfid = context->fi_l_filsiz - offset + 1;
    fh2 = context->fi_l_begadr + 512*(offset-1);

    if (cli_present("FILE_ID") & 1) {
      /* if FILE_ID specified, get the header, and display */
      while (cli_get_value("FILE_ID",&dynstr) & 1) {
        if (ots$cvt_ti_l(&dynstr,&i) & 1) {
          fid[0] = i;
          fh2 = get_header(fid,fid,context,1,chan);
          if (fh2)
            if (search_header(&search,fh2,context,1,chan))
              display_header(&display,fid[0],fh2,context,1,chan);
        };
      };
    }
    else
    for (currfid=1;currfid<=lastfid;currfid++) {
      if (currfid>endfid) {
        /* okay, that should have been the last one in this segment.. */
        /* lets read in the next section of the file */
        offset = offset + pagecount;
        pagecount = opagecount;
        status=map_section(&filename,&defaultname,
                           &pagecount,&offset,&flag,&context);
        if (!(status & 1)) return(status);
        endfid = endfid + pagecount;
        fh2 = context->fi_l_begadr;
      };

      if (search_header(&search,fh2,context,0,chan))
        (void)display_header(&display,currfid,fh2,context,0,chan);

      /* okay, point to the next file header */
      ++fh2;
    };

    /* call it a day, clean up and go home.... */
    status=delete_section(&context);
    if (!(status & 1)) return(status);

  if (volume)
    status = get_next_device(&device,&device);
  else
    status = 0;
  }; /* while (status & 1) */

}
