globalvalue EXE$GL_KNOWN_FILES;
globalvalue EXE$GQ_KFE_LCKNAM;

#include lckdef
#include "kfe.h"

int count_kfds();
int count_kfes();
int output_all();

struct LockStatusBlock {
  short int status;
  short int reserved;
  long int lock_id;
  char lock_value[16]; /* optional */
};

struct Info {
    char type;
    char buff[252];
    short int count; /* kfd$w_refcnt, or kfe$w_gblseccnt (if shared) */
    long int usecnt; /* kfe$l_usecnt */
    long int procpriv[2]; /* kfe$q_procpriv */
    union {
      short int flags;	/*   Flags word */
      struct {
        unsigned protect:1;	/*   Known file was installed protected */
        unsigned lim:1;		/*   Linkable image */
        unsigned procpriv:1;	/*   Use process privilege mask */
        unsigned open:1;	/*   Image installed /OPEN */
        unsigned hdrres:1;	/*   Image header block is resident */
        unsigned shared:1;	/*   Image is shared */
        unsigned shmident:1;	/*   Shared memory ident already set */
        unsigned compatmod:1;	/*   Image is compatability mode */
        unsigned nopurge:1;	/*   Image entry may not be purged */
        unsigned account:1;	/*   Image level accounting */
        unsigned writeable:1;	/*   Global sections are writeable */
        unsigned exeonly:1;	/*   Image has only execute access allowed */
      } flags_bits;
    } flags_overlay;
};

main(argc, argv)
  int argc;
  char *argv[];
{
  int status;

  int kfds = 0, kfes = 0;
  int kfd_args[2] = {1,&kfds};
  int kfe_args[2] = {1,&kfes};
  int out_args[2] = {1,0};
  struct LockStatusBlock lsb;
  struct Info *out;
  int i;

  /*
     According to the internals manual, I should take out a protected
     read lock on EXE$GQ_KFE_LCKNAM to access the KFD/KFE lists.
     I have not checked the microfiche yet to see if this is correct,
     but for now I am assuming this is about right.
  */

  /* Enque a protected read lock */
  status = sys$enqw(0,LCK$K_PRMODE,&lsb,LCK$M_SYSTEM,
		EXE$GQ_KFE_LCKNAM,0,0,0,0,0,0);
  if (status & 1) status = lsb.status;
  if (!(status & 1))
    lib$signal(status);

  status = sys$cmkrnl(&count_kfds,kfd_args);
  printf("%d - %d kfds\n",status, kfds);

  status = sys$cmkrnl(&count_kfes,kfe_args);
  printf("%d - %d kfes\n",status,kfes);

  out = malloc( (kfds + kfes) * sizeof(struct Info) );

  out_args[1] = out;
  status = sys$cmkrnl(&output_all,out_args);

  /* I should $DEQ the lock here... */
  status = sys$deq(lsb.lock_id,0,0,0);
  if (!(status & 1))
    lib$signal(status);

  for (i = 0; i < (kfds + kfes); ++i)
  {
    switch (out->type)
    {
      case 1:
        printf("%s (%d files)\n",out->buff,out->count);
        break;
      case 2:
        printf("\t%s, %d/%d%s%s%s%s%s%s%s%s%s%s%s%s",
               out->buff,
	       /* out->flags_overlay.flags_bits.shared?out->count:0, */
	       out->count,
               out->usecnt,
	       out->flags_overlay.flags_bits.protect?",protect":"",
	       out->flags_overlay.flags_bits.lim?",linkable":"",
	       out->flags_overlay.flags_bits.procpriv?",privileged":"",
	       out->flags_overlay.flags_bits.open?",open":"",
	       out->flags_overlay.flags_bits.hdrres?",header":"",
	       out->flags_overlay.flags_bits.shared?",shared":"",
	       out->flags_overlay.flags_bits.shmident?",ident set":"",
	       out->flags_overlay.flags_bits.compatmod?",compatability mode":"",
	       out->flags_overlay.flags_bits.nopurge?",not purgable":"",
	       out->flags_overlay.flags_bits.account?",accounting":"",
	       out->flags_overlay.flags_bits.writeable?",writeable":"",
	       out->flags_overlay.flags_bits.exeonly?",execute only":"");
          if (out->flags_overlay.flags_bits.procpriv)
          {
            printf (",priv mask = %08X %08X",out->procpriv[0],out->procpriv[1]);
          }
          printf("\n");
        break;
    }
    ++out;
  }
}

count_kfds(kfds)
  int *kfds;
{
  struct kfpbdef *kfpb = *(int *)EXE$GL_KNOWN_FILES; /* find kfpb */
  struct kfddef *kfd = kfpb->kfpb$l_kfdlst; /* find first kfd */

  *kfds = 0;
  while (kfd)
  {
    ++(*kfds);
    kfd = kfd->kfd$l_link; /* move on to the next kfd */
  }
  return(*kfds);
}

count_kfes(kfes)
  int *kfes;
{
  struct kfpbdef *kfpb = *(int *)EXE$GL_KNOWN_FILES; /* find kfpb */
  struct kfddef *kfd = kfpb->kfpb$l_kfdlst; /* find first kfd */
  struct kfedef *kfe;

  *kfes = 0;
  while (kfd)
  {
    kfe = kfd->kfd$l_kfelist; /* find first kfe for this kfd */
    while (kfe)
    {
      ++(*kfes);
      kfe = kfe->kfe$l_kfelink; /* move on to the next kfe */
    }
    kfd = kfd->kfd$l_link; /* move on to the next kfd */
  }
  return(*kfes);
}

output_all(out)
  struct Info *out;
{
  struct kfpbdef *kfpb = *(int *)EXE$GL_KNOWN_FILES; /* find kfpb */
  struct kfddef *kfd = kfpb->kfpb$l_kfdlst; /* find first kfd */
  struct kfedef *kfe;

  while (kfd)
  {
    out->type = 1;
    memcpy(out->buff, &(kfd->kfd$t_ddtstr), kfd->kfd$b_ddtstrlen);
    out->buff[kfd->kfd$b_ddtstrlen] = '\0';
    out->count = kfd->kfd$w_refcnt;
    ++out;

    kfe = kfd->kfd$l_kfelist; /* find first kfe for this kfd */
    while (kfe)
    {
      out->type = 2;
      memcpy(out->buff, &(kfe->kfe$t_filnam), kfe->kfe$b_filnamlen);
      out->buff[kfe->kfe$b_filnamlen] = '\0';
      out->count = kfe->kfe$w_gblseccnt;
      out->usecnt = kfe->kfe$l_usecnt;
      out->flags_overlay.flags = kfe->kfe$r_flags_overlay.kfe$w_flags;
      out->procpriv[0] = kfe->kfe$q_procpriv[0];
      out->procpriv[1] = kfe->kfe$q_procpriv[1];
      ++out;

      kfe = kfe->kfe$l_kfelink; /* move on to the next kfe */
    }
    kfd = kfd->kfd$l_link; /* move on to the next kfd */
  }
}
