/* IMAGE_INSTALLED determines whether the image name will be preceded by
 * an asterisk if the image is installed. If set to 1, the image name will
 * be preceded by an asterisk if the image name does not contain a version
 * number (indicating that it is installed). If set to anything else
 * the image name will not be preceded by anything.
 */
#define IMAGE_INSTALLED 0

/* IMAGE_CAPITALIZE determines whether the image name (return via IMAGE)
 * will be capitalized or not. If set to 1, the first letter will be
 * capitalized, and the first letter following any non-alphabetic
 * character will also be capitalized. All other characters will be
 * lowercased. If set to  anything else, the imagename will not be modified.
 */
#define IMAGE_CAPITALIZE 1

/* TIME_BLANKED determines whether time fields will have leading zeros
 * turned into blanks or not. If set to 1, all leading zeros will be
 * converted into blanks. All leading punctuation will also be removed
 * up to the first non zero digit. If set to anything else, the time fields
 * will not be modified.
 */
#define TIME_BLANKED 1

/* TERM_DATABASE determines whether a 'database' lookup will occur
 * to convert the terminal name. If set to 1, a database lookup
 * will occur (for the item WHERE). TERMFILE and TERMDFLT define
 * the name (and default filespec) of the terminal database.
 */
#define TERM_DATABASE 1
#define TERMFILE "STATUS_TERMINALS"
#define TERMDFLT "SYS$SYSTEM:.LOCATION"

/* WHO_DATABASE determines whether a 'database' lookup will occur
 * to find the personal name. If set to 1, a database lookup
 * will occur (for the item WHO). WHOFILE and WHODFLT define
 * the name (and default filespec) of the personal name database.
 * Note that this only occurs once per user. Once retrieved the name
 * is stored in an internal tree, which is referenced before bothering
 * to do a lookup. (This is because the code was there for the SYSUAF
 * lookup, so I figured, why not use it here too!)
 */
#define WHO_DATABASE 0
#define WHOFILE "STATUS_WHO"
#define WHODFLT "SYS$SYSTEM:.NAMES"

/* WHO_SYSUAF determines whether sys$getuai is used to retireve the
 * personal name for a user. If set to 1, sys$getuai will be called
 * to return the owner field for a given username (for the WHO item).
 * Note that this only occurs once per user. Once retrieved the name
 * is stored in an internal tree, which is referenced before bothering
 * to do a sys$getuai. This is there because sys$getuai is so (very) slow.
 */
#define WHO_SYSUAF 1
/* note, if both WHO_DATABASE and WHO_SYSUAF are set to 1, then a database
 * lookup is done first, if nothing is returned then a lookup in the sysuaf
 * is done.
 */

/* WHO_FIXNAME determines whether personal names retrieved from the
 * SYSUAF owner field (as above, via WHO_SYSUAF) are to be 'fixed'.
 * If set to 1, then a name with an imbedded comma will be 'fixed'.
 * If there is a comma, the name is assumed to be "Last, First",
 * and will be rewritten as "First Last".
*/
#define WHO_FIXNAME 1

/* SECNODE_DATABASE defines whether the node database is searched
 * to retrieve an access level for a client. If set to 1, the database
 * will be searched.
 * SECNODE and SECNODEDFLT define the name of the database file used
 * to look up the classification level of a node.
 * These are used in SECURE.C
 */
#define SECNODE_DATABASE 0
#define SECNODE "STATUS_NODES"
#define SECNODEDFLT "SYS$SYSTEM:.LEVEL"

/* SECUSER_DATABASE defines whether the node/user database is searched
 * to retrieve an access level for a client. If set to 1, the database
 * will be searched.
 * SECUSER and SECUSERDFLT define the name of the database file used
 * to look up the classification level of a node/user.
 * These are used in SECURE.C
 */
#define SECUSER_DATABASE 0
#define SECUSER "STATUS_USERS"
#define SECUSERDFLT "SYS$SYSTEM:.LEVEL"
