#include ctype
#include descrip
#include dvidef
#include "server.h"

#include "config.h" /* configures how this will compile */

#if WHO_SYSUAF == 1
static struct Tree {
  char *username;
  char *personal;
  struct Tree *left, *right;
} uaf_head = {0,0,0,0};

#endif

globalvalue EXE$GL_ABSTIM;

final_format(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  int pad,size,i,j;
  struct FmtItems *item;

  item = &(items->item[itm]);

  size = strlen(string);
  if (item->start) { /* was an offset specified? */
    if (item->start < 0) { /* was it from right? */
      size += item->start; /* remember, item->start < 0 */
      if (size < 0) size = 0;
      string[size] = '\0'; /* 'offset' in from right (truncate end) */
    } else { /* offset was from left */
      if (item->start < size) {
        size -= item->start;
        strcpy(string,&string[item->start]); /* chop off the first chars */
      } else {
        size = 0;
        string[0] = '\0';
      }
    }
  }

  if (item->width > 0) { /* string needs to be padded out to fit */
    pad = item->width - size;
    if (pad) {
      if (pad > 0) {
        switch (item->just) {
          case -1: /* add spaces to end */
            for (i=size; i < (size + pad); string[i++] = ' ');
            size += pad;
            string[size] = '\0';
            break;
          case 0: /* add spaces on both ends */
            i = pad / 2;
            strcpy(&string[i],string);
            for (j = 0; j < i; string[j++] = ' ');
            size += i;
            pad -= i;
            for (i=size; i < (size + pad); string[i++] = ' ');
            size += pad;
            string[size] = '\0';
            break;
          case 1: /* add spaces to front */
            strcpy(&string[pad],string);
            for (i = 0; i < pad; string[i++] = ' ');
            size += i;
            break;
        }
      } else { /* string is too big (pad < 0) */
        pad *= -1;
        switch (item->just) {
          case -1: /* truncate string to fit */
            string[size - pad] = '\0';
            size -= pad;
            break;
          case 0:  /* cut from both ends */
            i = pad / 2;
            string[size - i] = '\0'; /* truncate end */
            strcpy(string,&string[pad - i]); /* move over to start */
            size -= pad;
            break;
          case 1:  /* cut off start of string */
            strcpy(string,&string[pad]);
            size -= pad;
            break;
        }
      }
    }
  }
}

/* the following is used for literals, (keywords[KEYWORDS] points to it) */
format_literal(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  item = &(items->item[itm]);

  sprintf(string,"%.*s",item->width,get_address(item,link));
}

cvt$organization(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  strcpy(string,getenv("STATUS_ORGANIZATION"));
}

cvt$access_mode(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *modes[] = {"Kernel","Executive","Supervisor","User"};
  int *asts, i, comma = 0;

  item = &(items->item[itm]);

  asts = get_address(item,link);
  for (i=0; i<4; i++) {
    if ((*asts >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,modes[i]);
      string += strlen(string);
      comma = 1;
    }
  }
  if (! comma) {
    strcpy(string,"none");
  };
}

cvt$boolean(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int val;

  item = &(items->item[itm]);
  val = *(int *)get_address(item,link);

  strcpy(string,val?"true":"false");
}

cvt$cputim (items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  $DESCRIPTOR(date,"!%D");
  struct dsc$descriptor time = {23,DSC$K_DTYPE_T,DSC$K_CLASS_S,string};
  int val,cputim[2];
  short int len = 0;
  static int factor = -100000, zero = 0;

  item = &(items->item[itm]);
  val = *(int *)get_address(item,link);
  lib$emul(&factor,&val,&zero,cputim);
  sys$fao(&date,&len,&time,cputim);
  string[len] = '\0';
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

cvt$creprc_flags(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int flags, i, comma = 0;
  char *creflg[] = { "ssrwait", "ssfexcu", "pswapm", "noacnt", "batch",
                     "hiber", "nouaf", "netwrk", "disaws", "detach",
                     "inter", "imgdmp", "clispec", "nopassword" };

  item = &(items->item[itm]);
  flags = *(int *)get_address(item,link);

  for (i=0; i<14; i++) {
    if ((flags >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,creflg[i]);
      string += strlen(string);
      comma = 1;
    }
  }
  if (! comma) {
    strcpy(string,"none");
  }
}

cvt$elapsed_time(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  $DESCRIPTOR(date,"!%D");
  struct dsc$descriptor time = {23,DSC$K_DTYPE_T,DSC$K_CLASS_S,string};
  int logintim[2] = {0,0}, now[2];
  short int len = 0;

  item = &(items->item[itm]);
  memcpy(logintim, get_address(item,link),8);
  sys$gettim(now);
  lib$subx(logintim,now,logintim);
  sys$fao(&date,&len,&time,logintim);
  string[len] = '\0';
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

cvt$epid_to_ipid(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  item = &(items->item[itm]);

  sprintf(string,"%08X",epid_to_ipid(get_address(item,link)));
}

cvt$hex_6(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned short int val[3] = {0,0,0};
  item = &(items->item[itm]);

  memcpy(val, get_address(item,link), 6);
  sprintf(string,"%04X%04X%04X", val[0], val[1], val[2]);
}

cvt$hex_8(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned int val[2] = {0,0};
  item = &(items->item[itm]);

  memcpy(val, get_address(item,link), 8);
  sprintf(string,"%08X%08X", val[0], val[1]);
}

cvt$hex_12(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned int val[3] = {0,0,0};
  item = &(items->item[itm]);

  memcpy(val, get_address(item,link), 12);
  sprintf(string,"%08X%08X%08X", val[0], val[1], val[2]);
}

cvt$image(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int i,j, pid;
  char *s, *semi;
  char user[20];

  item = &(items->item[itm]);

  /* "trim" off trailing spaces and nulls */
  s = get_address(item,link);
  i = item->size - 1;

  /* pull off the .EXE; part */
  while ((i > 0) && (s[i] != '.'))
    --i;

  /* get rid of the disk and directory */
  j = i;
  while ((j > 0) && (s[j] != ']'))
    --j;

  if (s[j] == ']') ++j;

#if IMAGE_INSTALLED == 1
  semi = strchr(s,';');
  if (strlen(s) == 0)
    semi = 1; /* fake not being installed */
  sprintf(string,"%s%.*s",semi?" ":"*",i - j,&s[j]);
  ++string;
#else
  sprintf(string,"%.*s",i - j,&s[j]);
#endif

  /* if image = RTPAD, let's display where they are going! */
  if (strcmp("RTPAD",string) == 0)
  {
    /* grab the PID, and call Get_RTPAD_Destination */
    pid = *(int *)system_jpi.pcb_items[1].bufadr;
    Get_RTPAD_Destination(pid,&string[6],user);
    if (strlen(&string[6]))
    {
      /* a string was returned, so, let's surround it with () */
      string[5] = '(';
      if (strlen(user))
      {
        strcat(string,"::");
        strcat(string,user);
      }
      strcat(string,")");
    }
  }
  else
  {
    if (strcmp("TELNET",string) == 0)
    {
      /* grab the PID, and call Get_TELNET_Destination */
      pid = *(int *)system_jpi.pcb_items[1].bufadr;
      Get_TELNET_Destination(pid,&string[7]);
      if (strlen(&string[7]))
      {
        /* a string was returned, so, let's surround it with () */
        string[6] = '(';
        strcat(string,")");
      }
    }
  }
#if IMAGE_CAPITALIZE == 1
  capitalize(string);
#endif
}

cvt$jobtype(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  static char *jobtype[] = { "Detached", "Network", "Batch",
                             "Local", "Dialup", "Remote" };

  item = &(items->item[itm]);
  strcpy(string,jobtype[*(unsigned char *)get_address(item,link)]);
}

cvt$type(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int *jobtyp, *mode, *owner, typ;
  static char *jobtype[] = { "Detached", "Network", "Batch",
                             "Local", "Modem", "Remote" };

  item = &(items->item[itm]);
  jobtyp = get_address(&(items->item[itm]));
  mode = get_address(&(items->item[itm+1]));
  owner = get_address(&(items->item[itm+2]));

  typ = *jobtyp;
  if ((*jobtyp == 0) && (*mode == 1))
    typ = 1;

  strcpy(string,(*owner)?"Subprocess":jobtype[typ]);
}

cvt$mode(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  static char *mode[] = { "Other", "Remote", "Batch", "Interactive" };

  item = &(items->item[itm]);
  strcpy(string,mode[*(unsigned char *)get_address(item,link)]);
}

cvt$msgmsk(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *msg[] = { "text", "ident", "severity", "facility" };
  int flags, i, comma = 0;

  item = &(items->item[itm]);

  flags = *(int *)get_address(item,link);

  for (i=0; i<4; i++)
    if ((flags >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,msg[i]);
      string += strlen(string);
      comma = 1;
    }
  if (! comma)
    strcpy(string,"none");
}

cvt$numeric(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned long int l;

  item = &(items->item[itm]);

  memcpy(&l,get_address(item,link),item->size);
  sprintf(string,"%d",l);
}

cvt$numeric_hex(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned long int l;

  item = &(items->item[itm]);

  memcpy(&l,get_address(item,link),item->size);
  sprintf(string,"%0*X",item->size * 2,l);
}

cvt$numeric_oct(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned long int l;

  item = &(items->item[itm]);

  memcpy(&l,get_address(item,link),item->size);
  sprintf(string,"%0*o",item->size * 8 / 3,l);
}

cvt$pages(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  int ppgcnt, gpgcnt;
  struct FmtItems *item;

  ppgcnt = *(int *)get_address(&items->item[itm],link);
  gpgcnt = *(int *)get_address(&items->item[itm+1],link);
  sprintf(string,"%d",ppgcnt + gpgcnt);
}

cvt$io(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  int bufio, dirio;
  struct FmtItems *item;

  bufio = *(int *)get_address(&items->item[itm],link);
  dirio = *(int *)get_address(&items->item[itm+1],link);
  sprintf(string,"%d",bufio + dirio);
}

cvt$phdflags(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *phdflg[] = { "pfmflg", "dalcstx", "wspeakchk", "noaccvio",
                     "iwspeakck", "imgdmp", "no_ws_chng" };
  int flags, i, comma = 0;

  item = &(items->item[itm]);

  flags = *(int *)get_address(item,link);
  for (i=0; i < 7; i++)
    if ((flags >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,phdflg[i]);
      string += strlen(string);
      comma = 1;
    }
  if (! comma)
    strcpy(string,"none");
}

cvt$physical_terminal(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char buff[64];
  char node[128];
  char user[32];
  int status,len, i;
  struct itmlst dvi[2];
  struct dsc$descriptor devname = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0} ;
  struct TermInfo {
    long int duetim;
    long int pid;
    unsigned short int rtt_link;
  } terminfo;
  char *ptr,*start;

  item = &(items->item[itm]);

  dvi[0].length = sizeof(buff);
  dvi[0].code = DVI$_TT_PHYDEVNAM;
  dvi[0].bufadr = buff;
  dvi[0].retlen = &len;
  dvi[1].code = 0;
  dvi[1].length = 0;

  /* figure out length of string */
  len = item->size;
  start = get_address(item,link);
  ptr = start + len;
  while ((--ptr >= start) && ((*ptr == ' ') || (*ptr == '\0')))
    --len;

  ptr = start;
  if (*ptr == '_')
  {
    ++ptr;
    --len;
  };

  for (i=0; i < len; i++)
  {
    string[i] = *ptr++;
  };
  string[len] = '\0';

  if (string[len - 1] == ':')
    string[--len] = '\0';

  devname.dsc$w_length = len;
  devname.dsc$a_pointer = string;

  status = term_info(&devname,&terminfo); 	/* get rtt_link if remote */
  if ((status & 1) && terminfo.rtt_link)
  {
    for (i=0; i < 10; ++i) node[i] = '\0';
    /* here we need to call decnetinfo passing it the link number */
    status = Get_DECnet_Node(terminfo.rtt_link,node,user);
    if (status & 1)
    {
      strcpy(string,node);
      strcat(string,"::");
      strcat(string,user);
    }
  }
  else
  {
    if ((status = sys$getdviw(0,0,&devname,dvi,0,0,0,0)) & 1)
    {
      if (len)
      {
        buff[len] = '\0';
        if (buff[len - 1] == ':')
          buff[--len] = '\0';
        if (buff[0] == '_')
          sprintf(string,"%s",&buff[1]);
        else
          sprintf(string,"%s",buff);
        if ((strncmp(string,"TPA",3) == 0) || (strncmp(string,"TZA",3) == 0))
        {
          if (convert_pseudo_terminal(string,node) & 1)
          {
            strcpy(string,node);
          }
        }
      }
      else
      {
        sprintf(string,"Disconnected");
      }
    }
  }
}

cvt$privilege(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *privs[] = {"cmkrnl", "cmexec", "sysnam", "grpnam", "allspool",
                   "detach", "diagnose", "log_io", "group", "acnt",
                   "prmceb", "prmmbx", "pswapm", "altpri", "setprv",
                   "tmpmbx", "world", "mount", "oper", "exquota",
                   "netmbx", "volpro", "phy_io", "bugchk", "prmgbl",
                   "sysgbl", "pfnmap", "shmem", "sysprv", "bypass",
                   "syslck", "share", "upgrade", /* <- 32 not used */
                   "downgrade", /* <- 33 not used */ "grpprv", "readall",
                   "", "", /* <- 36 & 37 not used */ "security" };
  int priv0, priv1, i, comma = 0;

  item = &(items->item[itm]);

  priv0 = *(int *)get_address(item,link);
  for (i=0; i<32; i++)
    if ((priv0 >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,privs[i]);
      string += strlen(string);
      comma = 1;
    }

  priv1 = *(int *)(get_address(item,link) + 4);
  for (i=34; i<36; i++)
    if ((priv1 >> (i-32)) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,privs[i]);
      string += strlen(string);
      comma = 1;
    }

  if (! comma)
    strcpy(string,"none");
}

cvt$state(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  static char *states[] = {"", "colpg","mwait","cef","pfw","lef","lefo","hib",
                           "hibo","susp","suspo","fpg","com","como","cur" };

  static char *mwaits[] = {"unk","ast","mbx","npg","pff","pag","brk","img",
                           "quo","lck","swp","mpe","mpb","scs","clu"};
  int mutex,state;

  item = &(items->item[itm]);
  state = *(unsigned char *)get_address(item,link);

  state = *(int *)get_address(&items->item[itm],link);
  mutex = *(int *)get_address(&items->item[itm+1],link);

  if (state == 2)
  {
    if ((mutex < 15) && (mutex > 0))
    {
      strcpy(string,"rw");
      strcat(string,mwaits[mutex]);
    }
    else
    {
      sprintf(string,"mwait(%08X)",mutex);
    }
  }
  else
    strcpy(string,states[state]);
}

cvt$sts(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *sts[] = { "res", "delpen", "forcpen", "inquan", "pswapm", "respen",
                  "ssfexc", "ssfexce", "ssfexcs", "ssfexcu", "ssrwait",
                  "suspen", "wakepen", "wall", "batch", "noacnt", "swpvbn",
                  "astpen", "phdres", "hiber", "login", "netwrk", "pwrast",
                  "nodelet", "disaws", "inter", "recover", "secaudit" };
  int flag, i, comma = 0;

  item = &(items->item[itm]);
  flag = *(int *)get_address(item,link);
  for (i=0; i<27; i++)
    if ((flag >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,sts[i]);
      string += strlen(string);
      comma = 1;
    }
  if (! comma)
    strcpy(string,"none");
}

cvt$text(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int i;
  char *s;

  item = &(items->item[itm]);

  /* trim off trailing spaces and nulls */
  s = get_address(item,link);
  i = item->size - 1;
  while ((i >= 0) && ((s[i] == ' ') || (s[i] == '\0')))
    s[i--] = '\0';

  sprintf(string,"%.*s",i + 1,s);
}

cvt$nodename(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  int i;
  char *s;

  item = &(items->item[itm]);

  /* trim off trailing spaces and nulls */
  s = get_address(item,link);
  i = item->size - 1;
  while ((i >= 0) && ((s[i] == ' ') || (s[i] == '\0')))
    s[i--] = '\0';

  if (i < 0) /* gosh, there's no nodename.. */
  {
    strcpy(string,getenv("SYS$NODE"));
    i = strlen(string);
    while((i > 0) && (string[--i] == ':')) /* trim off trailing colons */
    {
      string[i] = '\0';
    }
  }
  else /* gosh, the nodename exists, copy it.. */
  {
    sprintf(string,"%.*s",i + 1,s);
  }
}

cvt$time(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  $DESCRIPTOR(date,"!%D");
  struct dsc$descriptor time = {23,DSC$K_DTYPE_T,DSC$K_CLASS_S,string};
  int bintim[2] = {0, 0};
  short int len = 0;

  item = &(items->item[itm]);
  memcpy(bintim, get_address(item,link), item->size);
  sys$fao(&date,&len,&time,bintim);
  string[len] = '\0';
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

cvt$uaf_flags(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char *uaf[] = { "disctly", "defcli", "lockpwd", "captive", "disacnt",
                  "diswelcom", "dismail", "nomail", "genpwd", "pwd_expired",
                  "pwd2_expired", "audit", "disreport", "disreconnect",
                  "autologin" };
  int flag, i, comma = 0;

  item = &(items->item[itm]);
  flag = *(int *)get_address(item,link);
  for (i=0; i<4; i++)
    if ((flag >> i) & 1) {
      if (comma) *string++ = ',';
      strcpy(string,uaf[i]);
      string += strlen(string);
      comma = 1;
    }
  if (! comma)
    strcpy(string,"none");
}

cvt$uic(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  static $DESCRIPTOR(uicfmt, "!%I");
  struct dsc$descriptor uicstr = {64,DSC$K_DTYPE_T,DSC$K_CLASS_S,string};
  short int len = 0;

  item = &(items->item[itm]);
  sys$fao(&uicfmt,&len,&uicstr,*(int *)get_address(item,link));
  string[len] = '\0';
}

cvt$where(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  char server[16], port[16];
  static int context = 0;

  item = &(items->item[itm]);
  cvt$physical_terminal(items,itm,string,link);

  /* see if it's a LT type terminal, if not, then just format it */
  if (strncmp(string,"LT",2) == 0) {
    if (readport(string ,server ,port) & 1) {
      strcpy(string, server);
      strcat(string, "/");
      strcat(string, port);
    }
  }

#if TERM_DATABASE == 1
  lookup_indexed(string,string,132,TERMFILE,TERMDFLT,&context);
#endif
}

cvt$who(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct dsc$descriptor user = {0,0,0,string};
#define UAI$_OWNER 12
  struct itmlst uailst[] = {32,UAI$_OWNER,&string[13],0,0,0,0,0};
  char *ptr;
  int length,status;
  static int context = 0;

  cvt$text(items,itm,string,link); /* trim username and such */
  status = 0;

#if WHO_DATABASE == 1
  status = lookup_user(&uaf_head,string,string);
  if (!(status & 1))
  {
    status = lookup_indexed(string,&string[14],128,WHOFILE,WHODFLT,&context);
    if (status & 1)
    {
      add_user(&uaf_head,string,&string[14]);
      strcpy(string,&string[14]);
    }
  }
#endif

#if WHO_SYSUAF == 1
  if (!status)
  {
    /* lookup user in local tree */
    status = lookup_user(&uaf_head,string,string);
    if (!(status & 1))
    {
      user.dsc$w_length = strlen(string);
      status = sys$getuai(0,0,&user,uailst,0,0,0);
      if (status & 1)
      {
        ptr = &string[14];
        ptr[string[13]] = '\0';
#if WHO_FIXNAME == 1
	fix_personal_name(ptr,ptr);
#endif
        add_user(&uaf_head,string,ptr);
        strcpy(string,ptr);
      }
    }
  }
#endif
}

cvt$global_word(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
{
  struct FmtItems *item;
  unsigned short int *wordptr;

  item = &(items->item[itm]);
  wordptr = get_address(item,link);

  sprintf(string,"%d",wordptr?*wordptr:0);
}

cvt$current_time(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  $DESCRIPTOR(date,"!%D");
  struct dsc$descriptor time = {23,DSC$K_DTYPE_T,DSC$K_CLASS_S,string};
  int now[2];
  short int len = 0;

  item = &(items->item[itm]);
  sys$gettim(now);
  sys$fao(&date,&len,&time,now);
  string[len] = '\0';
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

cvt$devices(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  static struct DevInfo {
    long int pid;
    short int len;
    char name[64];
  } devinfo[20];
  static int devinfosize = sizeof(struct DevInfo);
  static int class = -1, devinfocnt = 20;
  int *pid, status, i, indx;
  struct FmtItems *item;

  item = &(items->item[itm]);

  indx = 0;
  pid = get_address(item,link);

  /* find all devices owned by this PID */
  status = devices(&class, pid, &devinfocnt, &devinfosize, devinfo);
  if (status & 1) {
    /* okay, move all the device names into the tmp buffer */
    i = 0;
    while (devinfo[i].len) {
      /* strip the leading underscore and trailing colon while moving */
      indx += sprintf(&string[indx],"%.*s,",
			devinfo[i].len-2,&devinfo[i].name[1]);
      devinfo[i].len = 0;
      ++i;
    };
    if (indx) --indx;
  };
  string[indx] = '\0';
}

cvt$cluster_members(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct {
    short int BufferLength;
    short int ItemCode;
    long int BufferAddress;
    long int ReturnLengthAddress;
  } ItemList[]={15,SYI$_NODENAME,0,0,0,0,0,0};
  long int status = 1;
  long int iosb[2];
  long int csid = -1;
  char *node;
  int nodelen;

  node = string;

  while (status & 1)
  {
    nodelen = 0;
    ItemList[0].BufferAddress = node;
    ItemList[0].ReturnLengthAddress = &nodelen;
    status=sys$getsyiw(0,&csid,0,&ItemList,iosb,0,0);
    if (status & 1)
      status = iosb[0];
    if (status & 1)
    {
      node += nodelen;
      *node = ',';
      ++node;
    }
  }
  if (node > string)
    --node;
  *node = '\0';
}

cvt$idleterminal(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  struct dsc$descriptor device = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0} ;
  struct TermInfo {
    long int duetim;
    long int pid;
    short int rtt_link;
  } terminfo;
  int day,hrs,min,sec;
  int status;
  char *ptr;

  item = &(items->item[itm]);

  device.dsc$w_length = item->size;
  device.dsc$a_pointer = get_address(item,link);
  ptr = device.dsc$a_pointer + device.dsc$w_length;

  while ((ptr-- > device.dsc$a_pointer) && ((*ptr == ' ') || (*ptr == '\0')))
  {
    --device.dsc$w_length;
  }

  status = term_info(&device,&terminfo);
  if ((status & 1) && (terminfo.duetim > 0))
  {
    min = terminfo.duetim / 60;
    hrs = min / 60;
    day = hrs / 24;
    sec = terminfo.duetim - (min * 60);
    min = min - (hrs * 60);
    hrs = hrs - (day * 24);
    sprintf(string,"%4d %02d:%02d:%02d.00", day, hrs, min, sec);
  }
  else
  {
    if (status & 1)
    {
      sprintf(string,"   0 00:00:00.00");
    }
    else
    {
#ifdef DEBUG
printf("cvt$idletime - error %d from term_info\n",status);
#endif
      string[0] = '\0';
    }
  };
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

cvt$idlecpu(items,itm,string,link)
  struct FmtItemsList *items;
  int itm;
  char *string;
  int link;
{
  struct FmtItems *item;
  unsigned int now,time,day,hrs,min,sec;
  int status;
  char *ptr;

  item = &(items->item[itm]);

  memcpy(&time,get_address(item,link)+item->size,4);
  now = *(int *)EXE$GL_ABSTIM;
  time = now - time;

  if (time > 0)
  {
    min = time / 60;
    hrs = min / 60;
    day = hrs / 24;
    sec = time - (min * 60);
    min = min - (hrs * 60);
    hrs = hrs - (day * 24);
    sprintf(string,"%4d %02d:%02d:%02d.00", day, hrs, min, sec);
  }
  else
  {
    sprintf(string,"   0 00:00:00.00");
  };
#if TIME_BLANKED == 1
  convert_leading_zeros(string);
#endif
}

convert_leading_zeros(string)
  char *string;
{
  int i = 0,blanking = -1;

  while(string[i] && blanking)
  {
    if ((string[i] == '0') || (string[i] == ':') || (string[i] == '.'))
      string[i] = ' ';
    if (isdigit(string[i]))
    {
      while(string[i] && (string[i] != ' '))
        ++i;
      if (string[i] != ' ')
        blanking = 0;
    }
    ++i;
  }
}

capitalize(string)
  char *string;
{
  int i = 1;

  while (string[i])
  {
    if (isalpha(string[i - 1]))
      string[i] = tolower(string[i]);
    ++i;
  }
}

get_address(item,link)
  struct FmtItems *item;
  int link;
{
  int addr, slot;

  switch (item->flags.type) {
    case FLAGTYPEABS:
      return(item->address);
    case FLAGTYPEJPI:
      addr = system_jpi.data;
      slot = *(int *)system_jpi.pcb_items[0].bufadr;
      addr += (JPIMAXBUFFLEN + (4 * JPIMAXITEMCNT)) * slot;
      return(item->address + addr);
    case FLAGTYPESYI:
      addr = syi_items[link][0].bufadr;
      return(item->address + addr);
  }
}

lookup_user(node,user,name)
  struct Tree *node;
  char *user, *name;
{
  int cmp;

  if (node && node->username)
  {
    cmp = strcmp(user,node->username);
    if (cmp == 0)
    {
      strcpy(name,node->personal);
      return(1);
    };
    if (cmp < 0)
    {
      return(lookup_user(node->left,user,name));
    }
    else
    {
      return(lookup_user(node->right,user,name));
    }
  }
  else
  {
    return(0);
  }
}

add_user(node,user,name)
  struct Tree *node;
  char *user, *name;
{
  int cmp;

  if (node->username == 0) /* this is where it must go! */
  {
    node->username = malloc(strlen(user) + 1);
    node->personal = malloc(strlen(name) + 1);
    strcpy(node->username,user);
    strcpy(node->personal,name);
    return(1);
  }

  cmp = strcmp(user,node->username);
  if (cmp == 0)
  {
    /* hey, we already added this sucker, don't do repeat business */
    return(0);
  }

  if (cmp < 0)
  {
    if (node->left)
    {
      return(add_user(node->left,user,name));
    }
    else
    { /* this is where it goes! */
      node->left = calloc(1,sizeof(struct Tree));
      return(add_user(node->left,user,name));
    }
  }
  else
  {
    if (node->right)
    {
      return(add_user(node->right,user,name));
    }
    else
    { /* this is where it goes! */
      node->right = calloc(1,sizeof(struct Tree));
      return(add_user(node->right,user,name));
    }
  }
}

fix_personal_name(from,to)
  char *from, *to;
{
  char *comma, tmp[80];
  int len;

  while (*from == ' ')
    ++from; /* kill leading spaces */
  len = strlen(from) - 1;
  while (from[len] == ' ')
    from[len--] = '\0'; /* kill trailing spaces */

  /* look for a comma, if there, rearrange the name */
  comma = strchr(from,',');
  if (comma)
  {
    *comma++ = '\0';

    while (*comma == ' ')
      ++comma; /* kill leading spaces in first name */
    len = strlen(comma) - 1;
    while (comma[len] == ' ')
      comma[len--] = '\0'; /* kill trailing spaces in first name */

    /* copy strings together, first last */
    sprintf(tmp,"%s %s",comma,from);
    /* and output it.. */
    strcpy(to,tmp);
  }
  else
  {
    strcpy(to,from);
  };
}
