#include descrip
#include syidef

#include "config.h"

security_level(ncb)
  char *ncb;
{
  char node[32], user[32]; /* maybe they'll expand these some day?! */
  char *s1, *s2;
  int level;

  s1 = strchr(ncb,':');
  copy_string(node,ncb,s1 - ncb - 1);

  s2 = strchr(s1,'='); ++s2;
  s1 = strchr(s2,'/'); --s1;
  while(*s1 == ' ') --s1;
  copy_string(user,s2,s1 - s2);

  /*
  at this point the string 'user' contains the username of the client
  and the string 'node' contains the nodename of the client.

  You need to decide how you want to implement your security, this
  code is simply an example, although I have tried to make it reasonable.
  */

  /* see if this node/user has been given special access (or NO access!) */
  if ((level = lookup_user_classification(node,user)) == -1)
  { /* user not specifically granted any special value (including 0) */
    /* see if this node has been given special access (or NO access!) */
    if ((level = lookup_node_classification(node)) == -1)
    { /* this node not specifically given any special value (including 0) */
      if (same_node(node))
      {
        level = 3; /* really good access for people on this node */
      }
      else
      {
        if (same_cluster(node))
        {
          level = 2; /* decent access for cluster members */
        }
        else
        {
          level = 1; /* minimal access to other, non-member nodes */
        }
      }
    }
  }   
  return(level);
}

same_node(node)
  char *node;
{
  /* return 1 if the node is this node, 0 if not */
  long int iosb[2];
  char name[16];
  short int namelen;
  struct {
    short int BufferLength;
    short int ItemCode;
    long int BufferAddress;
    long int ReturnLengthAddress;
  } LocalList[]={15,SYI$_NODENAME,name,&namelen,0,0,0,0};

  sys$getsyiw(0,0,0,&LocalList,iosb,0,0);

  /* trim trailing spaces */
  name[namelen] = '\0';
  while(namelen && (name[--namelen] == ' '))
    name[namelen] = '\0';

  /* if namelen = 0, then use getenv("SYS$NODE") */
  if (namelen == 0)
  {
    strcpy(name,getenv("SYS$NODE"));
    /* trim off trailing colons */
    namelen = strlen(name);
    while(namelen && (name[--namelen] == ':'))
      name[namelen] = '\0';
  }

  return(strcmp(name,node) == 0);
}

same_cluster(node)
  char *node;
{
  /* return 1 if the node is a member of this cluster, 0 if not */
  struct dsc$descriptor nodename = {strlen(node),0,0,node};
  char member = 0;
  long int status, iosb[2];
  struct {
    short int BufferLength;
    short int ItemCode;
    long int BufferAddress;
    long int ReturnLengthAddress;
  } OtherList[]={1,SYI$_CLUSTER_MEMBER,&member,0,0,0,0,0};

  sys$getsyiw(0,0,&nodename,&OtherList,iosb,0,0);

  return(member);
}

lookup_node_classification(node)
  char *node;
{
  /* do some kind of lookup to decide the security level of the node */
  static int context = 0;
  char level[132];
  int val,status;

  val = -1; /* -1 means no record for this node was found */
#if SECNODE_DATABASE == 1
  status = lookup_indexed(node,level,sizeof(level),
                          SECNODE,SECNODEDFLT,&context);
  if (status & 1)
    val = atoi(level);
#endif

  return(val);
}

lookup_user_classification(node,user)                                     
  char *node, *user;
{
  /* do some kind of lookup to decide the security level of the node/user */
  static int context = 0;
  char nodeuser[65];
  char level[132];
  int val,status;

  val = -1; /* -1 means no record for this node was found */
#if SECUSER_DATABASE == 1
  strcpy(nodeuser,node);
  strcat(nodeuser,"/");
  strcat(nodeuser,user);

  status = lookup_indexed(nodeuser,level,sizeof(level),
                          SECUSER,SECUSERDFLT,&context);
  if (status & 1)
    val = atoi(level);
#endif

  return(val);
}

copy_string(dest,src,len)
  char *dest,*src;
  int len;
{
  dest[++len] = '\0';
  while(--len >= 0)
    dest[len] = toupper(src[len]);
}
